import moment from 'moment';
import 'moment-timezone';

import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';
import { MatSelectChange, MatSelect } from '@angular/material/select';

import { NativeService, NativeType } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { environment } from '@environments';

import {
  Settings,
  GeneralSetting,
  NotificationSetting,
  ChatSetting,
  PresenceSetting
} from '@app/models/settings';
import { I18nService } from '@ucap/ng-i18n';
import { VirtualScrollViewportComponent } from '@ucap/ng-ui';
import { FormControl } from '@angular/forms';
import { MatOptionSelectionChange } from '@angular/material/core';

export interface TimezoneData {
  displayName: string;
  name: string;
}

export interface SettingsDialogData {
  settings: Settings;
}
export interface SettingsDialogResult {}

@Component({
  selector: 'app-sections-account-settings',
  templateUrl: './settings.dialog.component.html',
  styleUrls: ['./settings.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SettingsDialogComponent implements OnInit, OnDestroy {
  @ViewChild('vsTimezone', { static: false })
  vsTimezone: VirtualScrollViewportComponent;

  @ViewChild('selectForTimezone', { static: false })
  selectForTimezone: MatSelect;

  formControlForTimezone = new FormControl();

  platform: 'browser' | 'electron' = 'electron';

  generalSetting: GeneralSetting;
  notificationSetting: NotificationSetting;
  chatSetting: ChatSetting;
  presenceSetting: PresenceSetting;

  timezoneList: TimezoneData[];
  timezonePlaceholder: string;

  supportedLanguages = environment.productConfig.supportedLanguages;
  supportedHrLanguages =
    environment.productConfig.organization.supportedLanguages;

  constructor(
    public dialogRef: MatDialogRef<SettingsDialogData, SettingsDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: SettingsDialogData,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private i18nService: I18nService,
    private changeDetectorRef: ChangeDetectorRef,
    public matDialog: MatDialog
  ) {
    this.nativeService.platform_nativeType().then((type) => {
      switch (type) {
        case NativeType.Browser:
          this.platform = 'browser';
          break;
        case NativeType.Electron:
          this.platform = 'electron';
          break;
        default:
          break;
      }
    });

    this.generalSetting = data.settings.general;
    this.notificationSetting = data.settings.notification;
    this.chatSetting = data.settings.chat;
    this.presenceSetting = data.settings.presence;
  }

  private ngOnDestroySubject: Subject<void> = new Subject();

  ngOnInit(): void {
    this.generateTimezoneData();

    this.i18nService.languageChanged$
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((lang) => {
        this.generateTimezoneData();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onSelectionChangeLanguage(event: MatSelectChange) {}

  onSelectionChangeHrLanguage(event: MatSelectChange) {}

  onOpenedChangeTimezone(opened: boolean) {
    if (opened) {
      this.setTimezoneData();
      this.vsTimezone.checkViewportSize();
    }
  }

  onSelectionChangeTimezone(event: MatOptionSelectionChange) {
    if (!event.isUserInput) {
      return;
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onCancel() {}

  onConfirm() {}

  private generateTimezoneData() {
    const timezoneData = this.i18nService.t('locale:timezone', {
      returnObjects: true
    });

    let timezoneList: TimezoneData[] = [];
    for (const name of moment.tz.names()) {
      const displayName = `(UTC${moment.tz(name).format('Z')}) ${
        timezoneData[name]
      }`;
      timezoneList.push({
        displayName,
        name
      });
      if (name === this.generalSetting.timezone) {
        this.timezonePlaceholder = displayName;
      }
    }
    timezoneList = timezoneList.sort((a: TimezoneData, b: TimezoneData) => {
      return a.displayName.localeCompare(b.displayName);
    });

    this.timezoneList = timezoneList;
  }

  private setTimezoneData() {
    const timezoneIndex = this.timezoneList.findIndex(
      (t) => t.name === this.generalSetting.timezone
    );

    if (-1 !== timezoneIndex) {
      if (!!this.vsTimezone && !!this.selectForTimezone) {
        this.vsTimezone.scrollToIndex(timezoneIndex);
        this.selectForTimezone.value = this.timezoneList[timezoneIndex].name;
      }
    }
  }
}
