import moment from 'moment';

import { Subject, of } from 'rxjs';
import { takeUntil, filter, take, map, catchError } from 'rxjs/operators';

import { Component, ViewChild, OnDestroy, OnInit } from '@angular/core';
import {
  Router,
  RouterEvent,
  NavigationEnd,
  PRIMARY_OUTLET,
  ActivatedRoute,
  Params
} from '@angular/router';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';
import { MatMenuTrigger } from '@angular/material/menu';
import { MatTabChangeEvent, MatTabGroup } from '@angular/material/tabs';
import { MatSidenav } from '@angular/material/sidenav';

import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfoSS } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';
import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { RoomSelector } from '@ucap/ng-store-chat';

import { AppSelector } from '@app/store/state';
import { AppChatService } from '@app/services/app-chat.service';
import { QueryParams as ChatQueryParams } from '@app/pages/chat/types/params.type';
import { CreateDialogComponent } from '@app/sections/group/dialogs/create.dialog.component';
import { AppAccountService } from '@app/services/app-account.service';
import { QueryParams as OrganizationParams } from '@app/pages/organization/types/params.type';
import { User } from '@ucap/protocol-info';

const NAVS = ['group', 'chat', 'organization', 'message'];

@Component({
  selector: 'app-layouts-default',
  templateUrl: './default.layout.component.html',
  styleUrls: ['./default.layout.component.scss']
})
export class DefaultLayoutComponent implements OnInit, OnDestroy {
  @ViewChild('navTabGroup', { static: true })
  navTabGroup: MatTabGroup;

  @ViewChild('leftSidenav', { static: true })
  leftSidenav: MatSidenav;

  isShowLeftSideNav = false;

  @ViewChild('profileMenuTrigger', { static: true })
  profileMenuTrigger: MatMenuTrigger;

  showStatusbar = true;

  tabIndex: string;
  queryParams: Params;

  unreadCountChat = 0;

  /** FAB */
  fabButtonShow = true;
  fabUseCustomDefaultIcon = true; // default in this prj
  fabButtons: { icon: string; tooltip?: string; divisionType?: string }[];
  versionInfo2Res: VersionInfo2Response;
  user: User;

  moment = moment;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private appAccountService: AppAccountService,
    private appChatService: AppChatService,
    private logService: LogService,
    public dialog: MatDialog
  ) {
    this.setFabInitial(NAVS[0]);
    this.router.events
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        filter((event) => event instanceof RouterEvent)
      )
      .subscribe((event) => {
        switch (event.constructor) {
          case NavigationEnd:
            {
              const t = this.router.parseUrl((event as NavigationEnd).url);
              const p = t.root.children[PRIMARY_OUTLET];
              if (!p || !p.segments || 0 === p.segments.length) {
                break;
              }
              const index = p.segments[0].path;
              this.setTabGroup(index);
              this.setFabInitial(index);
            }
            break;
          default:
            break;
        }
      });

    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params) => {
        this.queryParams = params;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(AppSelector.windowSize))
      .subscribe((size) => {
        if (size.width < 780) {
          if (this.leftSidenav.opened) {
            this.leftSidenav.close();
          }
          this.isShowLeftSideNav = false;
          this.leftSidenav.mode = 'over';
        } else {
          if (!this.leftSidenav.opened) {
            this.leftSidenav.open();
          }
          this.isShowLeftSideNav = true;
          this.leftSidenav.mode = 'side';
        }
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(RoomSelector.unreadTotal)
      )
      .subscribe((unreadTotal) => {
        this.unreadCountChat = unreadTotal;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS) {
    this.profileMenuTrigger.openMenu();
  }

  onSelectedTabChange(event: MatTabChangeEvent) {
    const commands: any = [
      NAVS[event.index],
      { outlets: { content: 'index' } }
    ];
    const orgInitialParams: Params = {};

    if (
      event.index === 1 && // is chat.
      !!this.queryParams &&
      !!this.queryParams[ChatQueryParams.ROOM_ID]
    ) {
      // 다른 화면에서 채팅으로 바로 유입할 경우에는 navigate 초기화를 무시한다.
      this.queryParams = undefined;
      return;
    }
    // if (!!this.tabIndex && this.tabIndex === 'chat') {
    //   if (!!this.queryParams && !!this.queryParams[ChatQueryParams.ROOM_ID]) {
    //     return;
    //   } else {

    //   }
    // } else {
    // }

    if (event.index === 2 && !!this.user) {
      orgInitialParams[OrganizationParams.DEPT_SEQ] = String(
        this.user.departmentCode
      );
    }
    this.router.navigate(commands, { queryParams: orgInitialParams });
    if (!this.isShowLeftSideNav) {
      this.leftSidenav.open();
    }
    this.setFabInitial(NAVS[event.index]);
  }

  onClickToggleLeftSidenav() {
    if (!this.isShowLeftSideNav) {
      this.leftSidenav.toggle();
    }
  }

  setFabInitial(type: string) {
    this.tabIndex = type;
    switch (type) {
      case 'group':
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'add',
              tooltip: '그룹 추가',
              divisionType: 'GROUP_NEW_ADD'
            }
          ];
        }
        break;
      case 'chat':
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'chat',
              tooltip: '대화 추가',
              divisionType: 'CAHT_NEW_ADD'
            }
          ];

          // if (environment.productConfig.CommonSetting.useTimerRoom) {
          //   this.fabButtons.push({
          //     icon: 'timer',
          //     tooltip: this.translateService.instant('chat.newTimerChat'),
          //     divisionType: 'CHAT_NEW_TIMER_ADD'
          //   });
          // }
        }
        break;
      case 'organization':
        {
          this.fabButtonShow = false;
        }
        break;
      case 'message':
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'add',
              tooltip: '쪽지 추가',
              divisionType: 'MESSAGE_NEW'
            }
          ];
        }
        break;
      // case MainMenu.Call:
      //   {
      //     this.fabButtonShow = false;
      //   }
      //   break;

      default: {
        this.fabButtonShow = false;
      }
    }
  }

  onClickFab(params: { btn: any }) {
    const btn = params.btn as {
      icon: string;
      tooltip?: string;
      divisionType?: string;
    };

    switch (btn.divisionType) {
      case 'GROUP_NEW_ADD':
        {
          const dialogRef = this.dialog.open(CreateDialogComponent, {
            panelClass: 'max-create-dialog'
          });

          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result) => {}),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        }
        break;
      case 'CAHT_NEW_ADD':
        {
          this.appChatService.newOpenRoomDialog();
        }
        break;
      case 'CHAT_NEW_TIMER_ADD':
        {
          // if (environment.productConfig.CommonSetting.useTimerRoom) {
          //   this.onClickNewChat('TIMER');
          // }
        }
        break;

      case 'MESSAGE_NEW':
        {
          this.logService.debug('MESSAGE_NEW');
        }
        break;
    }
  }

  onClickSearch(event: Event) {
    event.stopPropagation();
  }

  onSelectedProfileManage() {
    this.profileMenuTrigger.closeMenu();
  }

  onSelectedNotice() {
    this.profileMenuTrigger.closeMenu();
  }

  onSelectedSettings() {
    this.profileMenuTrigger.closeMenu();
    this.appAccountService.dialogForSettings();
  }

  onSelectedLogout() {
    this.router.navigate(['/account/logout']);
  }

  onSelectedExit() {}

  onDoneForProfileMenu() {
    this.profileMenuTrigger.closeMenu();
  }

  private setTabGroup(url: string) {
    if (!!this.navTabGroup) {
      this.navTabGroup.selectedIndex = NAVS.findIndex((v) => url === v);
    }
  }
}
