import { Component, ChangeDetectorRef, OnInit, OnDestroy } from '@angular/core';

import { LogService } from '@ucap/ng-logger';
import { Subject, of } from 'rxjs';
import { Store, select } from '@ngrx/store';
import { takeUntil, take, map, catchError } from 'rxjs/operators';
import { RoomSelector, RoomActions } from '@ucap/ng-store-chat';
import { RoomInfo, ExitAllRequest } from '@ucap/protocol-room';
import { MatDialog } from '@angular/material/dialog';
import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap/ng-ui';
import { I18nService } from '@ucap/ng-i18n';
import { SearchInfo } from '../models/search-info';
import { ActivatedRoute, Params } from '@angular/router';
import { QueryParams } from '../types/params.type';
import { AppChatService } from '@app/services/app-chat.service';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { AppKey } from '@app/types';

@Component({
  selector: 'app-pages-chat-sidenav',
  templateUrl: './sidenav.page.component.html',
  styleUrls: ['./sidenav.page.component.scss']
})
export class SidenavPageComponent implements OnInit, OnDestroy {
  searchObj: SearchInfo = {
    isShowSearch: false,
    searchWord: ''
  };

  checkable = false;

  historyRoomId: string;

  roomList: RoomInfo[];
  selectedRoomList: RoomInfo[] = [];
  searchResultList: RoomInfo[] = [];

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private logService: LogService,
    private sessionStorageService: SessionStorageService,
    private activatedRoute: ActivatedRoute,
    private appChatService: AppChatService,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.historyRoomId = this.sessionStorageService.get<string>(
      AppKey.HistoryRoomId
    );
  }

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ROOM_ID];

        this.sessionStorageService.set<string>(
          AppKey.HistoryRoomId,
          !!seqParam ? seqParam : undefined
        );

        if (
          seqParam === undefined &&
          !!this.historyRoomId &&
          this.historyRoomId.length > 0
        ) {
          // exist history roomId.
          this.appChatService.openRoombyRoomId(this.historyRoomId);
        }
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(RoomSelector.rooms))
      .subscribe((rooms) => {
        rooms = (rooms || []).filter((info) => info.isJoinRoom);
        this.roomList = rooms;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onToggleChackable(checkable: boolean): void {
    if (!!checkable) {
      if (!!this.selectedRoomList && this.selectedRoomList.length > 0) {
        const self = this;
        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('chat:label.exitFromRoom'),
            html: this.i18nService.t('chat:dialog.confirmExitFromRoom')
          }
        });

        dialogRef
          .afterClosed()
          .pipe(take(1))
          .subscribe((result) => {
            if (!!result && !!result.choice) {
              this.store.dispatch(
                RoomActions.delMulti({
                  req: {
                    roomIds: this.selectedRoomList.map(
                      (roomInfo) => roomInfo.roomId
                    )
                  } as ExitAllRequest
                })
              );
              self.selectedRoomList = [];
              self.checkable = false;
            }
          });
      }
    } else {
      this.selectedRoomList = [];
    }

    this.checkable = checkable;
  }

  getCheckedAllItem(): boolean {
    const targetRoomList = !!this.searchObj.isShowSearch
      ? this.searchResultList
      : this.roomList;

    if (
      !targetRoomList ||
      targetRoomList.length === 0 ||
      targetRoomList.filter(
        (item) =>
          !(
            this.selectedRoomList.filter((info) => info.roomId === item.roomId)
              .length > 0
          )
      ).length > 0
    ) {
      return false;
    } else {
      return true;
    }
  }

  onToggleAllItem(value: boolean): void {
    if (!!value) {
      const targetRoomList = !!this.searchObj.isShowSearch
        ? this.searchResultList
        : this.roomList;

      this.selectedRoomList = targetRoomList.slice();
    } else {
      this.selectedRoomList = [];
    }
    this.changeDetectorRef.markForCheck();
  }

  onToggleItem(event: { checked: boolean; roomInfo: RoomInfo }): void {
    if (!!event.checked) {
      if (
        !this.selectedRoomList.some(
          (info) => info.roomId === event.roomInfo.roomId
        )
      ) {
        this.selectedRoomList.push(event.roomInfo);
      }
    } else {
      if (!!this.selectedRoomList && this.selectedRoomList.length > 0) {
        const index = this.selectedRoomList.findIndex(
          (info) => info.roomId === event.roomInfo.roomId
        );
        if (index > -1) {
          this.selectedRoomList.splice(index, 1);
        }
      }
    }
  }

  /** Searching */
  onKeyDownSearch(params: { searchWord: string }) {
    this.searchObj = {
      isShowSearch: true,
      searchWord: params.searchWord
    } as SearchInfo;
    this.changeDetectorRef.markForCheck();
  }
  /** Searching cancel */
  onClickCancel() {
    this.searchObj = {
      isShowSearch: false,
      searchWord: ''
    } as SearchInfo;
    this.changeDetectorRef.markForCheck();
  }

  onSearchResultList(searchResultList: RoomInfo[]) {
    this.searchResultList = searchResultList;
  }
}
