import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ChangeDetectionStrategy
} from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';

import { MatDialog } from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import { FileUploadItem } from '@ucap/api';
import { FileProfileSaveRequest } from '@ucap/api-common';
import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfoSS, UserInfoF } from '@ucap/protocol-query';
import { UserInfoUpdateType, User } from '@ucap/protocol-info';
import { LoginResponse } from '@ucap/protocol-authentication';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';

import { UserInfoTypes } from '@app/types';
import { AppFileService } from '@app/services/app-file.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppGroupService } from '@app/services/app-group.service';

import { QueryParams } from '../types/params.type';
import { AppChatService } from '@app/services/app-chat.service';

@Component({
  selector: 'app-pages-group-index',
  templateUrl: './index.page.component.html',
  styleUrls: ['./index.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class IndexPageComponent implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private router: Router,
    private logService: LogService,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private appFileServie: AppFileService,
    private appAuthenticationService: AppAuthenticationService,
    private changeDetectorRef: ChangeDetectorRef,
    private appChatService: AppChatService,
    private appGroupService: AppGroupService
  ) {
    this.profileName = this.i18nService.t('organization:profile.me');
    this.tabName = this.i18nService.t('organization:profile.chatWithUs');
  }

  userSeq: string = undefined;
  user: User;
  loginRes: LoginResponse;
  versionInfo2Res: VersionInfo2Response;
  activeLink = 0;
  tabName: string;
  profileName: string;

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ID];
        if (!!seqParam) {
          this.userSeq = seqParam;
        } else {
          this.userSeq = '';
        }
        this._refreshProfile();
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this._refreshProfile();
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginRes))
      .subscribe((loginRes) => {
        this.loginRes = loginRes;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenCaht(userInfo: UserInfoSS) {
    this.appChatService.newOpenRoom(
      [Number(userInfo.seq) as any],
      false,
      this.user
    );
  }
  onSendMessage(userInfo: UserInfoSS) {}
  onSendCall(call: string) {}
  onSendSms(employeeNum: string) {}
  onCreateConference(userSeq: number) {}
  onToggleFavorit(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.appGroupService.updateBuddy(params.userInfo, params.isFavorite);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    this.appGroupService
      .updateBuddyByToggle(params)
      .then((isRemoveBuddy) => {
        if (isRemoveBuddy) {
          this.router.navigate(
            [
              'group',
              {
                outlets: { content: 'index' }
              }
            ],
            {
              queryParams: { id: Number(params.userInfo.seq) }
            }
          );
        }
      })
      .catch((reson) => this.logService.error(reson));
  }
  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    const loginSession = this.appAuthenticationService.getLoginSession();

    const profile = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      token: this.loginRes.tokenString,
      file: profileImageFileUploadItem.file,
      fileUploadItem: profileImageFileUploadItem
    } as FileProfileSaveRequest;

    this.appFileServie.fileProfileSave(
      profile,
      this.versionInfo2Res.profileUploadUrl
    );
  }
  onUpdateIntro(intro: string) {
    this.appGroupService.updateIntro(intro, UserInfoUpdateType.Intro);
  }

  onUpdateNickname(params: { userInfo: UserInfoTypes; nickname: string }) {
    this.appGroupService.updateNickname(
      params.userInfo as UserInfoF,
      params.nickname
    );
  }

  private _refreshProfile() {
    if (!this.user || undefined === this.userSeq) {
      return;
    }
    if ('' === this.userSeq) {
      this.userSeq = String(this.user.info.seq);
    }

    if (String(this.user.info.seq) === String(this.userSeq)) {
      this.profileName = this.i18nService.t('organization:profile.me');
      this.tabName = this.i18nService.t('organization:profile.unreadChat');
    } else {
      this.profileName = this.i18nService.t('organization:profile.other');
      this.tabName = this.i18nService.t('organization:profile.chatWithUs');
    }
    this.changeDetectorRef.markForCheck();
  }
}
