import moment from 'moment';

import { Subject, merge, combineLatest, Observable } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  AfterViewInit,
  ViewChild,
  ElementRef
} from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { MatDialog, MatDialogRef } from '@angular/material/dialog';

import { TransMassTalkDownloadRequest } from '@ucap/api-common';
import { VersionInfo2Response } from '@ucap/api-public';
import {
  Info,
  EventJson,
  EventType,
  FileType,
  MassTranslationEventJson
} from '@ucap/protocol-event';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo,
  UpdateTimerSetRequest,
  UpdateRequest,
  RoomType
} from '@ucap/protocol-room';
import { LoginResponse } from '@ucap/protocol-authentication';
import { FileInfo } from '@ucap/protocol-file';
import { User } from '@ucap/protocol-info';

import { I18nService } from '@ucap/ng-i18n';

import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';
import {
  ChattingSelector,
  RoomSelector,
  ChattingActions,
  RoomActions,
  Chatting
} from '@ucap/ng-store-chat';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  ClipboardService,
  SelectFileInfo
} from '@ucap/ng-ui';

import { LoginSession } from '@app/models/login-session';
import { AppChatService } from '@app/services/app-chat.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';

import {
  ForwardDialogComponent,
  ForwardDialogData,
  ForwardDialogResult
} from '../dialogs/forward.dialog.component';
import {
  FileViewerDialogComponent,
  FileViewerDialogData,
  FileViewerDialogResult
} from '../dialogs/file-viewer.dialog.component';
import {
  SettingDialogComponent,
  SettingDialogData,
  SettingDialogResult
} from '../dialogs/setting.dialog.component';

@Component({
  selector: 'app-sections-chat-message',
  templateUrl: './message.section.component.html',
  styleUrls: ['./message.section.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class MessageSectionComponent
  implements OnInit, OnDestroy, AfterViewInit {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  set roomId(roomId: string) {
    this._roomId = roomId;

    this.roomIdSubject.next(roomId);

    this.initializeRoomData();
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  translationSimpleview: boolean;

  @Input()
  eventSendTrigger$: Observable<any>;

  versionInfo2Res: VersionInfo2Response;
  loginRes: LoginResponse;
  user: User;
  loginSession: LoginSession;

  currentRoomInfo: RoomInfo;
  currentChatting: Chatting;
  currentFileInfoList: FileInfo[] = [];
  roomUsers: RoomUserInfoShort[] = [];
  // eventList$: Observable<Info<EventJson>[]>;
  eventList: Info<EventJson>[];

  defaultProfileImage: string;
  EventType = EventType;
  FileType = FileType;

  /** Timer 대화방의 대화 삭제를 위한 interval */
  interval: any;

  /** new Chat room setting dialog */
  settingDialogRef: MatDialogRef<SettingDialogComponent, SettingDialogResult>;

  /** About Scroll */
  isInitScrollbottom = true;
  @ViewChild('chatMessagesContainer', { static: false })
  chatMessagesContainer: ElementRef<HTMLElement>;

  String = String;

  constructor(
    private appChatService: AppChatService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private appAuthenticationService: AppAuthenticationService
  ) {
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => (this.versionInfo2Res = versionInfo2Res));

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginRes))
      .subscribe((loginRes) => (this.loginRes = loginRes));

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => (this.user = user));

    this.eventSendTrigger$
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((_) => {
        // Fires when I enter the event
        this.gotoScrollToBottom();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }

    if (!!this.interval) {
      clearInterval(this.interval);
      this.interval = undefined;
    }
  }

  ngAfterViewInit() {}

  initializeRoomData() {
    /** About initialize roomId */
    this.isInitScrollbottom = true;

    this.loginSession = this.appAuthenticationService.getLoginSession();

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(RoomSelector.room, this.roomId)
      )
      .subscribe((room) => {
        this.currentRoomInfo = room;

        // About Interval
        if (!!this.interval) {
          clearInterval(this.interval);
          this.interval = undefined;
        }
        if (!!this.currentRoomInfo && !!this.currentRoomInfo.isTimeRoom) {
          this.interval = setInterval(() => {
            this.store.dispatch(
              ChattingActions.intervalClearEvent({ roomId: this.roomId })
            );
          }, 1000);
        }
      });

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(RoomSelector.roomUserShort, this.roomId)
      )
      .subscribe((roomUserShort) => {
        if (!!roomUserShort) {
          this.roomUsers = roomUserShort.userInfos;
          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(ChattingSelector.chatting, this.roomId)
      )
      .subscribe((chatting) => {
        this.currentChatting = chatting;
        if (
          !!chatting &&
          !!chatting.fileInfoList &&
          chatting.fileInfoList.ids.length > 0
        ) {
          const tempFileInfoList: FileInfo[] = [];
          const fileInfoDic = chatting.fileInfoList.entities as Dictionary<
            FileInfo
          >;

          for (const key in fileInfoDic) {
            if (key === undefined) {
              continue;
            }

            if (fileInfoDic.hasOwnProperty(key)) {
              const fileInfo = fileInfoDic[key];
              tempFileInfoList.push(fileInfo);
            }
          }

          this.currentFileInfoList = tempFileInfoList;
        }
      });

    this.eventList = [];
    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(ChattingSelector.eventList, this.roomId)
      )
      .subscribe((eventList) => {
        if (!!eventList && eventList.length > 0) {
          this.eventList = eventList;
          this.changeDetectorRef.markForCheck();

          if (!!this.isInitScrollbottom) {
            this.gotoScrollToBottom();
            this.isInitScrollbottom = false;
          }
        }
      });

    /** Settings popup that opens when you first open a chat room */
    combineLatest([
      this.store.pipe(select(RoomSelector.room, this.roomId)),
      this.store.pipe(select(RoomSelector.standbyRooms))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([roomInfo, standbyRooms]) => {
        if (
          !!roomInfo &&
          roomInfo.roomId === this.roomId &&
          roomInfo.roomType !== RoomType.Mytalk &&
          roomInfo.roomType !== RoomType.Single &&
          !!standbyRooms &&
          standbyRooms.length > 0 &&
          standbyRooms.findIndex((roomId) => roomId === this.roomId) > -1
        ) {
          if (!this.settingDialogRef) {
            this.settingDialogRef = this.dialog.open<
              SettingDialogComponent,
              SettingDialogData,
              SettingDialogResult
            >(SettingDialogComponent, {
              panelClass: 'min-create-dialog',
              data: {
                roomId: this.roomId
              }
            });

            this.settingDialogRef
              .afterClosed()
              .pipe(take(1))
              .subscribe((result: SettingDialogResult) => {
                if (!!result && !!result.choice) {
                  this._standByRoomSetting(result);
                }
              });

            this.changeDetectorRef.markForCheck();
          }
        }
      });
  }

  gotoScrollToBottom() {
    if (!!this.chatMessagesContainer) {
      const self = this;
      setTimeout(() => {
        self.chatMessagesContainer.nativeElement.scrollTop =
          self.chatMessagesContainer.nativeElement.scrollHeight;
      }, 500);
    }
  }

  /** Sender Info */
  getSenderInfo(
    senderSeq: number
  ): RoomUserInfoShort | RoomUserInfo | undefined {
    if (!!this.roomUsers && this.roomUsers.length) {
      return this.roomUsers.find(
        (userInfo) => userInfo.seq + '' === senderSeq + ''
      );
    }

    return undefined;
  }

  /** Date Splitter show check */
  getDateSplitter(event: Info<EventJson>): boolean {
    const curIndex = this.eventList.findIndex((v) => v.seq === event.seq);

    if (curIndex === 0) {
      return true;
    }
    if (curIndex > 0) {
      if (!this.eventList[curIndex]) {
        return false;
      }
      return !moment(this.eventList[curIndex].sendDate).isSame(
        moment(this.eventList[curIndex - 1].sendDate),
        'day'
      );
    }
    return false;
  }

  /** Unread Count calculation */
  getUnreadCount(message: Info<EventJson>): string | number {
    const unreadCnt = this.roomUsers
      .filter(
        (user) => user.isJoinRoom && user.seq + '' !== message.senderSeq + ''
      )
      .filter((user) => user.lastReadEventSeq < message.seq).length;
    return unreadCnt === 0 ? '' : unreadCnt;
  }

  onClickShowPreviousEvents(event: MouseEvent) {
    event.preventDefault();
    event.stopPropagation();

    if (!!this.currentChatting.remainEvent) {
      this.store.dispatch(
        ChattingActions.moreEvents({
          roomId: this.roomId
        })
      );
    }
  }

  async onClickMessageContextMenu(params: {
    menuType: string;
    message: Info<EventJson>;
  }) {
    switch (params.menuType) {
      case 'COPY':
        {
          switch (params.message.type) {
            case EventType.Character:
              {
                this.appChatService.copyFromContentText(
                  (params.message as Info<string>).sentMessage
                );
              }
              break;
            case EventType.MassText:
              {
                this.appChatService.massTextDownload({
                  userSeq: String(this.user.info.seq),
                  deviceType: this.loginSession.deviceType,
                  token: this.loginRes.tokenString,
                  eventMassSeq: params.message.seq
                });
              }
              break;
            case EventType.Translation:
              {
              }
              break;
            case EventType.MassTranslation:
              {
              }
              break;
            default:
              break;
          }
        }
        break;
      case 'RECALL':
        {
          const dialogRef = this.dialog.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:label.recallEvent'),
              html: this.i18nService.t('chat:dialog.confirmRecallEvent')
            }
          });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.store.dispatch(
                  ChattingActions.cancel({
                    roomId: this.currentRoomInfo.roomId,
                    eventSeq: params.message.seq,
                    deviceType: this.loginSession.deviceType
                  })
                );
              }
            });
        }
        break;
      case 'FORWARD':
        {
          const dialogRef = this.dialog.open<
            ForwardDialogComponent,
            ForwardDialogData,
            ForwardDialogResult
          >(ForwardDialogComponent, {
            panelClass: 'max-create-dialog'
          });
          dialogRef
            .afterOpened()
            .pipe(take(1))
            .subscribe(() => {
              dialogRef.componentInstance.psUpdate();
            });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                const userSeqs: string[] = [];
                let roomSeq = '';
                if (
                  !!result.selelctUserList &&
                  result.selelctUserList.length > 0
                ) {
                  result.selelctUserList.map((user) =>
                    userSeqs.push(String(user.seq))
                  );
                }
                if (!!result.selectRoom) {
                  roomSeq = result.selectRoom.roomId;
                }
                if (userSeqs.length > 0 || roomSeq.trim().length > 0) {
                  this.store.dispatch(
                    ChattingActions.forward({
                      senderSeq: String(this.user.info.seq),
                      deviceType: this.loginSession.deviceType,
                      req: {
                        roomId: '-999',
                        eventType: params.message.type,
                        sentMessage: params.message.sentMessage
                      },
                      trgtUserSeqs: userSeqs,
                      trgtRoomId: roomSeq
                    })
                  );
                }
              }
            });
        }
        break;
      case 'FORWARD_TO_ME':
        {
          if (this.loginRes.talkWithMeBotSeq > -1) {
            const seqs = this.user.talkWithMeBotSeq as any;
            this.store.dispatch(
              ChattingActions.forward({
                senderSeq: String(this.user.info.seq),
                deviceType: this.loginSession.deviceType,
                req: {
                  roomId: '-999',
                  eventType: params.message.type,
                  sentMessage: params.message.sentMessage
                },
                trgtUserSeqs: [seqs]
              })
            );
          }
        }
        break;
      case 'DELETE':
        {
          const dialogRef = this.dialog.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:dialog.title.removeChat'),
              html: this.i18nService.t('chat:dialog.confirmRemoveChat')
            }
          });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.store.dispatch(
                  ChattingActions.del({
                    roomId: this.currentRoomInfo.roomId,
                    eventSeq: params.message.seq
                  })
                );
              }
            });
        }
        break;
    }
  }

  onMassTranslationDetail(params: {
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }) {
    const req = {
      userSeq: String(this.user.info.seq),
      token: this.loginRes.tokenString,
      deviceType: this.loginSession.deviceType,
      eventTransSeq: params.message.sentMessageJson.translationSeq.toString()
    } as TransMassTalkDownloadRequest;
    this.appChatService.massTranslationMessageDetail(
      req,
      params.message,
      params.contentsType,
      this.roomId
    );
  }

  onFileViewer(selectFileInfo: SelectFileInfo) {
    const data: FileViewerDialogData = {
      fileInfos: this.currentFileInfoList,
      selectFileInfo,
      downloadUrl: this.versionInfo2Res.downloadUrl,
      deviceType: this.loginSession.deviceType,
      token: this.loginRes.tokenString,
      userSeq: String(this.user.info.seq)
    };
    this.appChatService.openFileviwer(data);
  }

  private _standByRoomSetting(result: SettingDialogResult) {
    if (result.timerInterval > 0 && !!result.roomInfo.isTimeRoom) {
      this.store.dispatch(
        RoomActions.updateTimeRoomInterval({
          req: {
            roomId: result.roomInfo.roomId,
            timerInterval: result.timerInterval
          } as UpdateTimerSetRequest
        })
      );
    } else {
      this.store.dispatch(
        RoomActions.update({
          req: {
            roomId: result.roomInfo.roomId,
            roomName: result.roomName,
            receiveAlarm: result.roomInfo.receiveAlarm,
            syncAll: result.syncAll
          } as UpdateRequest
        })
      );
    }
  }

  onOpenProfile(userSeq: string): void {
    const result = this.dialog.open<
      ProfileDialogComponent,
      ProfileDialogData,
      ProfileDialogResult
    >(ProfileDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        userSeq
      }
    });
  }
}
