import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  Input,
  ViewChild
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import {
  UserInfoSS,
  UserInfoF,
  UserInfoDN,
  AuthResponse
} from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';
import { MatStepper } from '@angular/material/stepper';
import { I18nService } from '@ucap/ng-i18n';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui';
import { environment } from '@environments';
import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';
import { AuthorizationSelector } from '@ucap/ng-store-authentication';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

export interface CreateDialogData {}
export interface CreateDialogResult {
  userSeqs: string[];
  isTimer: boolean | undefined;
}

@Component({
  selector: 'app-dialog-chat-create',
  templateUrl: './create.dialog.component.html',
  styleUrls: ['./create.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CreateDialogComponent implements OnInit, OnDestroy {
  currentStep = 0;
  maxChatRoomUser: number;

  isTimer: boolean | undefined;
  selectedUserList: UserInfoTypes[] = [];
  authRes: AuthResponse;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<CreateDialogData, CreateDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: CreateDialogData,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.authResponse)
      )
      .subscribe((authRes) => {
        this.authRes = authRes;
        if (!!this.authRes && !this.authRes.canTimerChat) {
          this.isTimer = false;
          this.currentStep = 1;
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }
  onCancel(stepper: MatStepper) {
    if (!!this.authRes && !this.authRes.canTimerChat) {
      // close.
    } else if (stepper.selectedIndex > 0) {
      stepper.previous();
      return;
    }
    this.dialogRef.close();
  }
  onConfirm(stepper: MatStepper) {
    // validation.
    if (this.isTimer === undefined) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          html: this.i18nService.t('chat:errors.emptyOpenRoomType')
        }
      });
      return;
    }
    stepper.next();
  }

  onOpenRoom(stepper: MatStepper) {
    const userSeqs: string[] = [];

    this.selectedUserList.map((user) => userSeqs.push(user.seq.toString()));

    if (this.selectedUserList.length >= this.maxChatRoomUser) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          html: this.i18nService.t('chat:errors.maxCountOfRoomMemberWith', {
            maxCount: this.maxChatRoomUser
          })
        }
      });
      return;
    }

    // Open Room.
    this.dialogRef.close({ userSeqs, isTimer: this.isTimer });
  }

  onChangeUserList(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => String(u.seq) === String(d.userInfo.seq)
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => String(p.seq) === String(u.seq))
      );
    }
  }
  onChangeGroupList(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter(
            (user) => String(user.seq) === String(item.seq)
          ).length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter(
            (del) => String(del.seq) === String(item.seq)
          ).length === 0
      );
    }
  }
  onRemovedProfileSelection(userInfo: UserInfo) {
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
  }
  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };
  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
