import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo, RoomInfo } from '@ucap/protocol-room';

import { I18nService } from '@ucap/ng-i18n';

import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { GroupSelector } from '@ucap/ng-store-group';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { Expansion02Component as AppExpansion02Component } from '@app/ucap/group/components/expansion-02.component';
import { environment } from '@environments';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

export interface ForwardDialogData {}
export interface ForwardDialogResult {
  choice: boolean;
  selelctUserList?: UserInfoTypes[];
  selectRoom?: RoomInfo;
}

@Component({
  selector: 'app-dialog-chat-forward',
  templateUrl: './forward.dialog.component.html',
  styleUrls: ['./forward.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ForwardDialogComponent implements OnInit, OnDestroy {
  @ViewChild('appGroupExpansion', { static: false })
  appGroupExpansion: AppExpansion02Component;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }

    this.onChangedCompanySearch();
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  private ngOnDestroySubject: Subject<void> = new Subject();

  maxChatRoomUser: number;

  selectedRoom: RoomInfo;
  selectedUserList: UserInfoTypes[] = [];

  versionInfo2Res: VersionInfo2Response;

  groupList: GroupDetailData[];

  isSearch = false;
  searchedList: UserInfoSS[] = [];
  currentTabIndex: number;

  constructor(
    public dialogRef: MatDialogRef<ForwardDialogData, ForwardDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ForwardDialogData,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  psUpdate() {
    this.appGroupExpansion.psUpdate();
  }
  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ choice: false });
  }

  onSelectedIndexChange(value: number): void {
    this.currentTabIndex = value;
    this._resetSelectedObject();
  }

  private _resetSelectedObject(): void {
    this.selectedRoom = undefined;
    this.selectedUserList = [];
  }

  onSearched(searchedUserInfos: UserInfoSS[]): void {
    this.searchedList = searchedUserInfos;
  }

  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => u.seq === d.userInfo.seq
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
      );
    }
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter((user) => user.seq === item.seq)
            .length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter((del) => del.seq === item.seq)
            .length === 0
      );
    }
  }

  onCancel() {
    this.dialogRef.close({ choice: false });
  }
  onConfirm() {
    if (
      !this.selectedRoom &&
      !!this.selectedUserList &&
      this.selectedUserList.length === 0
    ) {
      const result = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          message: this.i18nService.t('chat:errors.inputChatMessage')
        }
      });
      return;
    }

    this.dialogRef.close({
      choice: true,
      selectRoom: this.selectedRoom,
      selelctUserList: this.selectedUserList
    });
  }

  onToggleRoom(roomInfo: RoomInfo): void {
    if (!!roomInfo) {
      this.selectedRoom = roomInfo;
    }
  }

  getCheckedByRoomInfo(roomInfo: RoomInfo): boolean {
    if (
      !!this.selectedRoom &&
      this.selectedRoom.roomId.localeCompare(roomInfo.roomId) === 0
    ) {
      return true;
    }

    return false;
  }
  onChangeUserList(data: { checked: boolean; userInfo: UserInfoSS }) {
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(data.userInfo.seq)
    );

    if (data.checked) {
      if (-1 === i) {
        this.selectedUserList = [...this.selectedUserList, data.userInfo];
      }
    } else {
      if (-1 < i) {
        this.selectedUserList = this.selectedUserList.filter(
          (u) => String(u.seq) !== String(data.userInfo.seq)
        );
      }
    }
  }

  onChangeGroupList(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter(
            (user) => String(user.seq) === String(item.seq)
          ).length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter(
            (del) => String(del.seq) === String(item.seq)
          ).length === 0
      );
    }
  }

  onChangedCompanySearch() {
    if (!!this.companySearchData && this.companySearchData.bySearch) {
      this.isSearch = true;
    }
  }

  getCheckedSearchAllItem(): boolean {
    const targetRoomList = this.searchedList;

    if (
      !targetRoomList ||
      targetRoomList.length === 0 ||
      targetRoomList.filter(
        (item) =>
          !(
            this.selectedUserList.filter(
              (info) => String(info.seq) === String(item.seq)
            ).length > 0
          )
      ).length > 0
    ) {
      return false;
    } else {
      return true;
    }
  }

  onToggleSearchAllItem(value: boolean): void {
    if (!!value) {
      const targetRoomList = this.searchedList;
      this.selectedUserList = targetRoomList.slice();
    } else {
      this.selectedUserList = [];
    }
    this.changeDetectorRef.markForCheck();
  }

  onCanceled() {
    this.isSearch = false;
    this.searchedList = [];
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
  }
  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };
  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
