import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ElementRef,
  AfterViewInit
} from '@angular/core';

import { Subject, of, combineLatest } from 'rxjs';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { NativeService } from '@ucap/native';
import { VersionInfo2Response } from '@ucap/api-public';

import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort
} from '@ucap/protocol-room';

import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { RoomSelector } from '@ucap/ng-store-chat';
import { Info, MassTranslationEventJson } from '@ucap/protocol-event';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;

export interface TransDetailDialogData {
  contents: string;
  message: Info<MassTranslationEventJson>;
  roomId: string;
  defaultProfileImage: string;
}
export interface TransDetailDialogResult {}

@Component({
  selector: 'app-dialog-chat-trans-detail',
  templateUrl: './trans-detail.dialog.component.html',
  styleUrls: ['./trans-detail.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TransDetailDialogComponent
  implements OnInit, OnDestroy, AfterViewInit {
  userInfo: UserInfoTypes;
  contents: string;

  defaultProfileImage: string;
  versionInfo2Res: VersionInfo2Response;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<
      TransDetailDialogData,
      TransDetailDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: TransDetailDialogData,
    public dialog: MatDialog,
    private store: Store<any>,
    private elementRef: ElementRef,
    private logService: LogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.defaultProfileImage = this.data.defaultProfileImage;
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });
    combineLatest([
      this.store.pipe(select(RoomSelector.roomUser, this.data.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.data.roomId))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([roomUsers, roomUsersShort]) => {
        const userList: UserInfoTypes[] =
          roomUsers?.userInfos || roomUsersShort?.userInfos;
        this.userInfo = userList.find(
          (item) => item.seq + '' === this.data.message.senderSeq + ''
        );
        this.changeDetectorRef.markForCheck();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {
    if (
      !!this.elementRef.nativeElement &&
      !!this.elementRef.nativeElement.querySelector('a')
    ) {
      const elements = this.elementRef.nativeElement.querySelectorAll('a');
      elements.forEach((element) => {
        element.addEventListener('click', this.onClickEvent.bind(this));
      });
    }
  }

  onClickEvent(event: MouseEvent) {
    this.nativeService.platform_openDefaultBrowser(
      (event.target as HTMLAnchorElement).text
    );
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }
}
