import { Subject, combineLatest, merge } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  Inject,
  NgZone
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LoginSession, DeviceType } from '@ucap/core';
import { NativeService } from '@ucap/native';
import { FileDownloadItem } from '@ucap/api';
import { VersionInfo2Response } from '@ucap/api-public';
import { LoginResponse } from '@ucap/protocol-authentication';
import { RoomInfo } from '@ucap/protocol-room';
import { FileInfo, InfoRequest } from '@ucap/protocol-file';
import { FileType, FileEventJson } from '@ucap/protocol-event';
import { User } from '@ucap/protocol-info';

import { I18nService } from '@ucap/ng-i18n';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';
import {
  RoomSelector,
  ChattingSelector,
  ChattingActions
} from '@ucap/ng-store-chat';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap/ng-ui';

import { ChatDrawType } from '@app/pages/chat/types/chat-draw.type';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppFileService } from '@app/services/app-file.service';
import { AppChatService } from '@app/services/app-chat.service';

import { FileViewerDialogData } from '../dialogs/file-viewer.dialog.component';

export type SendRecvType = 'ALL' | 'SEND' | 'RECV';

@Component({
  selector: 'app-drawer-chat-attach-data',
  templateUrl: './attach-data.drawer.component.html',
  styleUrls: ['./attach-data.drawer.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AttachDataDrawerComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject = new Subject<boolean>();

  @Input()
  set roomId(value: string) {
    this._roomId = value;
    this.roomIdSubject.next(value);
    this._initializeData();
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  set drawerType(type: ChatDrawType) {
    this._drawerType = type;
    this.filterSendRecvType = 'ALL';

    this.getFiltered();
  }
  get drawerType(): ChatDrawType {
    return this._drawerType;
  }
  // tslint:disable-next-line: variable-name
  _drawerType: ChatDrawType;

  @Output()
  closed = new EventEmitter<void>();

  originalFileInfoList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];
  filteredFileInfoList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];
  selectedFileInfoList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];

  filterSendRecvType: SendRecvType = 'ALL';

  versionInfo2Res: VersionInfo2Response;
  loginRes: LoginResponse;
  user: User;
  loginSession: LoginSession;

  roomInfo: RoomInfo;
  showDownCheckSeq: number | undefined;

  ChatDrawType = ChatDrawType;
  DeviceType = DeviceType;

  constructor(
    private store: Store<any>,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private appChatService: AppChatService,
    private appFileService: AppFileService,
    private appAuthenticationService: AppAuthenticationService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private ngZone: NgZone,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => (this.versionInfo2Res = versionInfo2Res));

    this.loginSession = this.appAuthenticationService.getLoginSession();

    this._initializeData();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  private _initializeData() {
    this.originalFileInfoList = [];
    this.filteredFileInfoList = [];
    this.selectedFileInfoList = [];
    this.showDownCheckSeq = undefined;

    combineLatest([
      this.store.pipe(select(LoginSelector.loginRes)),
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.room, this.roomId)),
      this.store.pipe(select(ChattingSelector.fileInfoList, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([loginRes, user, roomInfo, fileInfoList]) => {
        this.loginRes = loginRes;
        this.user = user;
        this.roomInfo = roomInfo;

        if (!!fileInfoList && fileInfoList.length > 0) {
          this.originalFileInfoList = fileInfoList
            .map((item) => {
              return {
                fileInfo: item,
                fileDownloadItem: new FileDownloadItem()
              };
            })
            .sort((a, b) =>
              a.fileInfo.eventSeq < b.fileInfo.eventSeq
                ? 1
                : a.fileInfo.eventSeq > b.fileInfo.eventSeq
                ? -1
                : 0
            );
          this.getFiltered();
        }
      });
  }

  onChangeDrawerType(type: ChatDrawType) {
    this.selectedFileInfoList = [];
    this.showDownCheckSeq = undefined;
    this.drawerType = type;
  }

  onChangeFilterSendRecv(type: SendRecvType) {
    this.selectedFileInfoList = [];
    this.filterSendRecvType = type;
    this.getFiltered();
  }

  getFiltered() {
    if (
      !!this.drawerType &&
      !!this.originalFileInfoList &&
      this.originalFileInfoList.length > 0
    ) {
      this.filteredFileInfoList = this.originalFileInfoList
        .filter((item) => {
          if (this.drawerType === ChatDrawType.AttachImage) {
            return item.fileInfo.type === FileType.Image;
          } else if (this.drawerType === ChatDrawType.AttachVideo) {
            return item.fileInfo.type === FileType.Video;
          } else {
            return (
              item.fileInfo.type !== FileType.Image &&
              item.fileInfo.type !== FileType.Video
            );
          }
        })
        .filter((item) => {
          if (this.filterSendRecvType === 'RECV') {
            return (
              String(item.fileInfo.senderSeq) !== String(this.user.info.seq)
            );
          } else if (this.filterSendRecvType === 'SEND') {
            return (
              String(item.fileInfo.senderSeq) === String(this.user.info.seq)
            );
          } else {
            return true;
          }
        });
    }

    this.changeDetectorRef.detectChanges();
  }

  refreshList(): void {
    let searchType: FileType = FileType.All;
    switch (this.drawerType) {
      case ChatDrawType.AttachImage:
        searchType = FileType.Image;
        break;
      case ChatDrawType.AttachVideo:
        searchType = FileType.Video;
        break;
      case ChatDrawType.AttachFile:
        searchType = FileType.File;
        break;
    }
    this.store.dispatch(
      ChattingActions.fileInfos({
        req: { roomId: this.roomId, type: searchType } as InfoRequest
      })
    );
  }

  onClosed(event: MouseEvent): void {
    this.closed.emit();
  }

  getCheckedAllItem(): boolean {
    const targetList = this.filteredFileInfoList;

    if (
      !targetList ||
      targetList.length === 0 ||
      targetList.filter(
        (item) =>
          !(
            this.selectedFileInfoList.filter(
              (info) => info.fileInfo.seq === item.fileInfo.seq
            ).length > 0
          )
      ).length > 0
    ) {
      return false;
    } else {
      return true;
    }
  }
  onToggleAllItem(value: boolean): void {
    if (!!value) {
      const targetList = this.filteredFileInfoList;

      this.selectedFileInfoList = this.filteredFileInfoList;
    } else {
      this.selectedFileInfoList = [];
    }
    this.changeDetectorRef.markForCheck();
  }
  onToggleItem(params: {
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }) {
    if (!!params.checked) {
      if (
        !this.selectedFileInfoList.some(
          (info) => info.fileInfo.seq === params.fileInfo.seq
        )
      ) {
        this.selectedFileInfoList.push({
          fileInfo: params.fileInfo,
          fileDownloadItem: params.fileDownloadItem
        });
      }
    } else {
      if (!!this.selectedFileInfoList && this.selectedFileInfoList.length > 0) {
        const index = this.selectedFileInfoList.findIndex(
          (info) => info.fileInfo.seq === params.fileInfo.seq
        );
        if (index > -1) {
          this.selectedFileInfoList.splice(index, 1);
        }
      }
    }
  }

  onShowDownCheck(fileInfo: FileInfo): void {
    this.showDownCheckSeq = fileInfo.seq;
  }

  onClickItemDelete(fileInfo: FileInfo): void {
    const dialogRef = this.dialog.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      panelClass: 'min-create-dialog',
      data: {
        title: this.i18nService.t('chat:dialog.title.removeChat'),
        html: this.i18nService.t('chat:dialog.confirmRemoveChat')
      }
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result) => {
        if (!!result && !!result.choice) {
          this.store.dispatch(
            ChattingActions.del({
              roomId: fileInfo.roomId,
              eventSeq: fileInfo.eventSeq
            })
          );
        }
      });
  }

  onClickItemOpenViewer(fileInfo: FileInfo): void {
    const data: FileViewerDialogData = {
      fileInfos: this.filteredFileInfoList.map((item) => item.fileInfo),
      selectFileInfo: { attachmentSeq: fileInfo.seq },
      downloadUrl: this.versionInfo2Res.downloadUrl,
      deviceType: this.loginSession.deviceType,
      token: this.loginRes.tokenString,
      userSeq: String(this.user.info.seq)
    };
    this.appChatService.openFileviwer(data);
  }
  onClickItemSave(params: {
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }): void {
    if (
      params.type === 'saveAs' &&
      this.loginSession.deviceType === DeviceType.PC
    ) {
      this.nativeService
        .file_selectForSave({ defaultPath: params.fileInfo.fileName })
        .then((result) => {
          if (!!result) {
            if (!!result.canceled) {
              // 취소함.
            } else {
              this.appFileService.saveFile(
                {
                  fileInfo: params.fileInfo,
                  fileDownloadItem: params.fileDownloadItem,
                  type: params.type,
                  fileName: params.fileInfo.fileName,
                  fileDownloadUrl: undefined,
                  savePath: result.filePath
                },
                this.loginRes,
                this.user,
                this.loginSession
              );
            }
          }
        })
        .catch((reason) => {
          // this.snackBarService.open(
          //   this.translateService.instant(
          //     'common:file.errors.failToSpecifyPath'
          //   ),
          //   this.translateService.instant('common:file.errors.label')
          // );
        });
    } else {
      this.appFileService.saveFile(
        {
          fileInfo: params.fileInfo,
          fileDownloadItem: params.fileDownloadItem,
          type: params.type,
          fileName: params.fileInfo.fileName,
          fileDownloadUrl: undefined,
          savePath: undefined
        },
        this.loginRes,
        this.user,
        this.loginSession
      );
    }
  }

  onClickDelete(): void {
    const dialogRef = this.dialog.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      panelClass: 'min-create-dialog',
      data: {
        title: this.i18nService.t('chat:dialog.title.removeChat'),
        html: this.i18nService.t('chat:dialog.confirmRemoveChat')
      }
    });

    const self = this;
    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result) => {
        if (!!result && !!result.choice) {
          self.selectedFileInfoList.forEach((item, i) => {
            setTimeout(() => {
              this.store.dispatch(
                ChattingActions.del({
                  roomId: item.fileInfo.roomId,
                  eventSeq: item.fileInfo.eventSeq
                })
              );
            }, i * 300);
          });
        }
      });
  }
  onClickChangeFolder(): void {
    alert('changeFolder');
  }
  onClickOpenViewer(): void {
    if (!!this.filteredFileInfoList && this.filteredFileInfoList.length > 0) {
      this.onClickItemOpenViewer(this.filteredFileInfoList[0].fileInfo);
    }
  }
  onClickDownload(): void {
    this.ngZone.run(() => {
      this.selectedFileInfoList.forEach((item) => {
        this.onClickItemSave({
          fileInfo: {
            // require parameters.
            fileName: item.fileInfo.name,
            attachmentSeq: item.fileInfo.seq
          } as FileEventJson,
          fileDownloadItem: item.fileDownloadItem,
          type: 'save'
        });
      });
    });
  }
}
