import { Subject, combineLatest, merge } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { VersionInfo2Response } from '@ucap/api-public';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  ExitForcingRequest
} from '@ucap/protocol-room';
import { User } from '@ucap/protocol-info';

import { I18nService } from '@ucap/ng-i18n';

import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { RoomSelector, RoomActions } from '@ucap/ng-store-chat';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap/ng-ui';

import { ChatDrawType } from '@app/pages/chat/types/chat-draw.type';
import { DrawInfo } from '@app/pages/chat/models/draw-info';
import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';
import { AppChatService } from '@app/services/app-chat.service';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;

@Component({
  selector: 'app-drawer-chat-users',
  templateUrl: './users.drawer.component.html',
  styleUrls: ['./users.drawer.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class UsersDrawerComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject = new Subject<boolean>();

  @Input()
  set roomId(value: string) {
    this._roomId = value;
    if (!!this.roomIdSubject) {
      this.roomIdSubject.next(value);
    }
    this._initializeData();
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Output()
  rightDrawerToggle = new EventEmitter<DrawInfo>();

  @Output()
  closed = new EventEmitter<void>();

  user: User;
  versionInfo2Res: VersionInfo2Response;

  roomUsers: RoomUserInfo[] = [];
  roomUsersShort: RoomUserInfoShort[] = [];

  constructor(
    private i18nService: I18nService,
    public dialog: MatDialog,
    private store: Store<any>,
    private appChatService: AppChatService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this._initializeData();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  private _initializeData() {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([user, roomUser, roomUserShort]) => {
        this.user = user;
        if (
          !!roomUser &&
          !!roomUser.userInfos &&
          roomUser.userInfos.length > 0
        ) {
          this.roomUsers = roomUser.userInfos.filter(
            (item) => !!item.isJoinRoom
          );
        }
        if (
          !!roomUserShort &&
          !!roomUserShort.userInfos &&
          roomUserShort.userInfos.length > 0
        ) {
          this.roomUsersShort = roomUserShort.userInfos.filter(
            (item) => !!item.isJoinRoom
          );
        }

        this.changeDetectorRef.markForCheck();
      });
  }

  onClosed(event: MouseEvent): void {
    this.closed.emit();
  }
  onClickAddUserChatRoom(): void {
    this.rightDrawerToggle.emit({
      chatDrawType: ChatDrawType.Invite,
      returnDrawType: ChatDrawType.RoomUsers
    });
  }
  onClickAddGroupMember(): void {
    this.rightDrawerToggle.emit({
      chatDrawType: ChatDrawType.AddGroup,
      returnDrawType: ChatDrawType.RoomUsers
    });
  }

  onClickUser(userInfo: UserInfoTypes): void {}

  onClickOpenProfile(userSeq: string): void {
    const result = this.dialog.open<
      ProfileDialogComponent,
      ProfileDialogData,
      ProfileDialogResult
    >(ProfileDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        userSeq
      }
    });
  }
  onClickOpenChatRoom(userSeq: string): void {
    this.appChatService.newOpenRoom([userSeq], false);
  }
  onClickSendMessage(userSeq: string): void {}

  onClickExitForcing(userInfo: UserInfoTypes): void {
    const dialogRef = this.dialog.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      data: {
        title: this.i18nService.t('chat:dialog.title.ejectFromRoom'),
        html: this.i18nService.t('chat:dialog.confirmEjectFromRoom', {
          targetMember: userInfo.name
        })
      }
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result) => {
        if (!!result && !!result.choice) {
          this.store.dispatch(
            RoomActions.expel({
              req: {
                roomId: this.roomId,
                type: 'A',
                senderSeq: String(this.user.info.seq),
                userSeqs: [userInfo.seq + '']
              } as ExitForcingRequest
            })
          );
        }
      });
  }

  isMe(userInfo: UserInfoTypes) {
    if (!!this.user) {
      return String(this.user.info.seq) === String(userInfo.seq);
    } else {
      return false;
    }
  }
}
