import { Subject, of } from 'rxjs';
import { takeUntil, take, map, catchError } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ViewChild,
  EventEmitter,
  Output,
  ElementRef
} from '@angular/core';

import { Router } from '@angular/router';

import { MatDialog } from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { User } from '@ucap/protocol-info';
import {
  UserInfoSS,
  UserInfoF,
  UserInfoDN,
  DeptSearchType
} from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { QueryProtocolService } from '@ucap/ng-protocol-query';

import {
  PresenceActions,
  PresenceSelector,
  UserSelector
} from '@ucap/ng-store-organization';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui';

import { Expansion01Component as AppExpansion01Component } from '@app/ucap/group/components/expansion-01.component';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { SearchData } from '@app/ucap/organization/models/search-data';
import { UserStore } from '@app/models/user-store';
import { AppChatService } from '@app/services/app-chat.service';
import { GroupManageType } from '@app/types';
import { AppGroupService } from '@app/services/app-group.service';
import { LoginSession } from '@app/models/login-session';
import { GroupOpenInfo } from '@app/models/group-open-info';

import {
  ManageDialogComponent,
  ManageDialogData,
  ManageDialogResult
} from '../dialogs/manage.dialog.component';

import {
  EditUserDialogComponent,
  EditUserDialogData,
  EditUserDialogResult
} from '../dialogs/edit-user.dialog.component';
import { SortViewType } from '@app/pages/group/types/sort-view.type';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

@Component({
  selector: 'app-sections-group-list',
  templateUrl: './list.section.component.html',
  styleUrls: ['./list.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ListSectionComponent implements OnInit, OnDestroy {
  @Input()
  set searchData(searchData: SearchData) {
    this._searchData = searchData;
    if (!searchData) {
      this._searchData = {
        companyCode: this.userStore.companyCode
      };
    } else {
      if (!this._searchData.companyCode) {
        this._searchData.companyCode = this.userStore.companyCode;
      }

      if (
        searchData.bySearch &&
        searchData.searchWord.localeCompare('') !== 0
      ) {
        this.onOrganizationTenantSearch(searchData);
      } else {
        this._searchData.isShowSearch = false;
        this.searchUserInfos = [];
      }
    }
  }

  get searchData() {
    return this._searchData;
  }
  // tslint:disable-next-line: variable-name
  _searchData: any;

  @Input()
  checkable = false;

  @Input()
  showType: SortViewType;

  @Output()
  checkUser = new EventEmitter<{
    isChecked: boolean;
    userInfo: any;
  }>();

  @Output()
  clickUser = new EventEmitter<UserInfo>();

  @ViewChild('appGroupExpansion01', { static: false })
  appGroupExpansion: AppExpansion01Component;

  @ViewChild('nicknameInput', { static: false })
  nicknameInput: ElementRef<HTMLInputElement>;

  loginSession: LoginSession;
  user: User;

  searchUserInfos: UserInfoSS[] = [];

  private ngOnDestroySubject: Subject<void> = new Subject();
  private userStore: UserStore;

  constructor(
    private router: Router,
    private appAuthenticationService: AppAuthenticationService,
    private sessionStorageService: SessionStorageService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    private logService: LogService,
    private queryProtocolService: QueryProtocolService,
    private dialog: MatDialog,
    private appChatService: AppChatService,
    private appGroupService: AppGroupService
  ) {
    this.userStore = this.appAuthenticationService.getUserStore();
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });
    this.appAuthenticationService
      .getLoginSession$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((loginSession) => {
        this.loginSession = loginSession;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOrganizationTenantSearch(searchData: SearchData) {
    const searchUserInfos: UserInfoSS[] = [];

    this.queryProtocolService
      .deptUser({
        divCd: 'GRP',
        companyCode: searchData.companyCode,
        searchRange: DeptSearchType.All,
        search: searchData.searchWord,
        senderCompanyCode: this.user.info.companyCode,
        senderEmployeeType: this.user.info.employeeType
      })
      .pipe(
        take(1),
        map((resObj) => {
          const userInfos = resObj.userInfos;

          searchUserInfos.push(...userInfos);
          // 검색 결과 처리.
          this.searchUserInfos = searchUserInfos.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          );
          // this.searchProcessing = false;

          // 검색 결과에 따른 프레즌스 조회.
          const userSeqList: string[] = [];
          this.searchUserInfos.map((user) => userSeqList.push(user.seq));
          this.changeDetectorRef.markForCheck();

          if (userSeqList.length > 0) {
            this.store.dispatch(
              PresenceActions.bulkInfo({
                divCd: 'groupSrch',
                userSeqs: userSeqList
              })
            );
          }
        }),
        catchError((error) => {
          // this.searchProcessing = false;
          return of(error);
        })
      )
      .subscribe();
  }

  onCheckUser(params: { isChecked: boolean; userInfo: any }) {
    this.checkUser.emit(params);
  }

  onClickUser(params: { event: MouseEvent; userInfo: UserInfo }) {
    if (!!params.event) {
      params.event.preventDefault();
      params.event.stopPropagation();
    }

    this.clickUser.emit(params.userInfo);
  }

  onClickSearchUser(event: MouseEvent, userInfo: UserInfo) {
    this.onClickUser({ event, userInfo });
  }
  onGroupDestroy(groupOpenInfo: GroupOpenInfo) {
    this.appAuthenticationService.setLoginSession({
      ...this.loginSession,
      groupInfo: groupOpenInfo
    });
  }
  onExpandMore() {
    this.appGroupExpansion.onExpandMore();
  }
  onExpandLess() {
    this.appGroupExpansion.onExpandLess();
  }

  onProfileMenu(event) {
    console.log(event);
  }
  onSelectGroupMenu(params: {
    menuType: string;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
    rect: any;
  }) {
    switch (params.menuType) {
      case 'CHAT':
        {
          this.appChatService.newOpenRoom(
            params.groupBuddyList.group.userSeqs,
            false,
            this.user
          );
        }
        break;
      case 'SEND_MESSAGE':
        {
          return;

          this.dialog.open<
            AlertDialogComponent,
            AlertDialogData,
            AlertDialogResult
          >(AlertDialogComponent, {
            data: {
              title: this.i18nService.t('group:dialog.title.messageGroup'),
              html: this.i18nService.t('group:dialog.title.messageGroup')
            }
          });
        }
        break;
      case 'RENAME':
        {
          this.renameGroup(params);
        }
        break;
      case 'MANAGE_MEMBER':
        {
          const dialogRef = this.dialog.open<
            ManageDialogComponent,
            ManageDialogData,
            ManageDialogResult
          >(ManageDialogComponent, {
            panelClass: 'max-create-dialog',
            data: {
              title: this.i18nService.t('group:dialog.title.managementGroup'),
              groupBuddyList: params.groupBuddyList
            }
          });
          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.appGroupService.addMemberToGroup(
                  result.group,
                  result.selelctUserList
                );
              }
            });
          dialogRef
            .afterOpened()
            .pipe(take(1))
            .subscribe(() => {
              dialogRef.componentInstance.psUpdate();
            });
        }
        break;
      case 'COPY_MEMBER':
        {
          this.editUserDialog(
            GroupManageType.Copy,
            params.groupBuddyList.group,
            params.groupBuddyList.buddyList
          );
        }
        break;
      case 'MOVE_MEMBER':
        {
          this.editUserDialog(
            GroupManageType.Move,
            params.groupBuddyList.group,
            params.groupBuddyList.buddyList
          );
        }
        break;
      case 'DELETE':
        {
          this.appGroupService.removeGroup(params.groupBuddyList);
        }
        break;
      default:
        break;
    }
  }

  onSelectProfileMenu(params: {
    menuType: string;
    userInfo: UserInfoF;
    group: GroupDetailData;
    rect: any;
  }) {
    switch (params.menuType) {
      case 'REGISTER_FAVORITE':
        this.appGroupService.updateBuddy(
          params.userInfo,
          !params.userInfo.isFavorit
        );
        break;
      case 'REGISTER_NICKNAME':
        {
          this.editNickname(params.userInfo, params.rect);
        }
        break;
      case 'COPY_BUDDY':
        this.editUserDialog('COPY_BUDDY', params.group, [params.userInfo]);
        break;
      case 'MOVE_BUDDY':
        this.editUserDialog('MOVE_BUDDY', params.group, [params.userInfo]);
        break;
      case 'REMOVE_BUDDY':
        {
          this.removeBuddy(params.userInfo, params.group);
        }
        break;
    }
  }

  onFloatingProfileMenu(params: { menuType: string; userInfo: UserInfoF }) {
    switch (params.menuType) {
      case 'CHAT':
        this.appChatService.newOpenRoom(
          [params.userInfo.seq as any],
          false,
          this.user
        );
        break;
      case 'MESSAGE':
        break;
      case 'MOBILE':
        break;
      case 'OFFICE':
        break;
      case 'VIDEO_CONFERENCE':
        break;
    }
  }

  private renameGroup(params: {
    menuType: string;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
    rect: any;
  }) {
    this.appGroupService.updateGroupName(
      params.groupBuddyList.group,
      params.rect
    );
  }

  private updateMemberByResult(result: EditUserDialogResult) {
    let targetGroup: GroupDetailData;
    let targetUserSeqs: string[];

    if (result.type === GroupManageType.Add) {
      this.appGroupService.addMemberToGroup(
        result.group,
        result.selelctUserList
      );
    } else if (result.type === GroupManageType.Copy) {
      if (!!result.selectGroupList && result.selectGroupList.length > 0) {
        result.selectGroupList.map((g) => {
          this.appGroupService.copyMemberToGroup(g, result.selelctUserList);
        });
      }
    } else if (result.type === GroupManageType.Move) {
      const fromGroup = result.group;
      let toGroup: GroupDetailData;

      if (!!result.selectGroupList && result.selectGroupList.length > 0) {
        result.selectGroupList.forEach((g, idx) => {
          toGroup = g;
          targetUserSeqs = [];

          if (!!result.selelctUserList && result.selelctUserList.length > 0) {
            if (idx === 0) {
              this.appGroupService.moveMemberToGroup(
                fromGroup,
                toGroup,
                result.selelctUserList
              );
            } else {
              this.appGroupService.copyMemberToGroup(
                toGroup,
                result.selelctUserList
              );
            }
          }
        });
      }
    } else if (result.type === GroupManageType.Create) {
      this.appGroupService.createGroup(
        result.groupName,
        result.selelctUserList
      );
    }
  }

  private editUserDialog(
    type: string,
    group: GroupDetailData,
    userInfos: UserInfoTypes[]
  ) {
    let title = '';
    let dialogType: GroupManageType;
    if (type === 'COPY_BUDDY') {
      dialogType = GroupManageType.Copy;
      title = this.i18nService.t('group:contextMenu.copyBuddy');
    } else if (type === 'MOVE_BUDDY') {
      dialogType = GroupManageType.Move;
      title = this.i18nService.t('group:contextMenu.moveBuddy');
    } else if (type === GroupManageType.Copy) {
      dialogType = GroupManageType.Copy;
      title = this.i18nService.t('group:dialog.title.copyGroup');
    } else if (type === GroupManageType.Move) {
      dialogType = GroupManageType.Move;
      title = this.i18nService.t('group:dialog.title.moveGroup');
    }
    const dialogRef = this.dialog.open<
      EditUserDialogComponent,
      EditUserDialogData,
      EditUserDialogResult
    >(EditUserDialogComponent, {
      panelClass: 'max-create-dialog',
      data: {
        title,
        type: dialogType,
        group,
        userInfos
      }
    });
    dialogRef
      .afterClosed()
      .pipe(
        take(1),
        map((result: EditUserDialogResult) => {
          this.updateMemberByResult(result);
        }),
        catchError((err) => {
          return of(err);
        })
      )
      .subscribe();
  }

  private removeBuddy(userInfo: UserInfoF, group: GroupDetailData) {
    this.appGroupService.removeMemberToGroup(
      this.i18nService.t('group:dialog.removeBuddyConfirm', {
        targetMember: `${userInfo.name}`
      }),
      [userInfo],
      group
    );
  }
  private editNickname(userInfo: UserInfoF, rect: any) {
    this.appGroupService.updateNickname(userInfo, undefined, rect);
  }

  onChangeNickname(nick: string) {}
}
