import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  ViewChild
} from '@angular/core';

import { MatDialog } from '@angular/material/dialog';
import { MatCheckbox, MatCheckboxChange } from '@angular/material/checkbox';

import { Store, select } from '@ngrx/store';

import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { GroupDetailData, UserInfo } from '@ucap/protocol-sync';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';

import { I18nService } from '@ucap/ng-i18n';

import { GroupSelector } from '@ucap/ng-store-group';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { ProfileListComponent as AppGroupProfileListComponent } from '@app/ucap/group/components/profile-list.component';
import {
  PerfectScrollbarDirective,
  PerfectScrollbarComponent
} from 'ngx-perfect-scrollbar';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

@Component({
  selector: 'app-sections-select-group',
  templateUrl: './select-group.section.component.html',
  styleUrls: ['./select-group.section.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SelectGroupSectionComponent
  implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  isMemberMove: boolean;

  @Input()
  isDialog = false;

  @Input()
  checkable = false;

  @Input()
  searchable = true;

  @Input()
  curGroup: GroupDetailData;

  @Input()
  set selectedUserList(userList: UserInfoTypes[]) {
    this._selectedUserList = userList;

    this._allCheckedBySearchUser();
  }
  get selectedUserList() {
    return this._selectedUserList;
  }
  // tslint:disable-next-line: variable-name
  _selectedUserList: UserInfoTypes[];

  @Input()
  selectedGroupList: GroupDetailData[] = [];

  @Input()
  groupName: string;

  @Output()
  changeUserList: EventEmitter<
    {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  > = new EventEmitter();

  @Output()
  changeGroupList: EventEmitter<{
    checked: boolean;
    group: GroupDetailData;
  }> = new EventEmitter();

  @Output()
  changeGroupName: EventEmitter<{
    invalid: boolean;
    groupName: string;
  }> = new EventEmitter();

  @ViewChild('groupProfileList', { static: false })
  groupProfileList: AppGroupProfileListComponent;

  @ViewChild('checkboxAllGroup', { static: false })
  checkboxAllGroup: MatCheckbox;

  @ViewChild('checkboxAllSearch', { static: false })
  checkboxAllSearch: MatCheckbox;

  @ViewChild(PerfectScrollbarDirective, { static: false })
  psDirectiveRef: PerfectScrollbarDirective;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }

    this.onChangedCompanySearch();
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private dialog: MatDialog,
    private i18nService: I18nService
  ) {}

  isSearch = false;
  searchWord: string;
  searchUserInfos: UserInfoSS[] = [];
  groupList: GroupDetailData[];

  groupChecked = false;

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        let defaultGroup: GroupDetailData;
        const buddyGroup: GroupDetailData[] = [];
        const tempOrder: GroupDetailData[] = [];

        groups.forEach((group) => {
          if (0 === group.seq) {
            defaultGroup = group;
          } else {
            if (!!this.curGroup && this.curGroup.seq === group.seq) {
              // ignore..
            } else {
              buddyGroup.push(group);
            }
          }
        });

        tempOrder.push(
          ...buddyGroup.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          )
        );

        if (!!defaultGroup) {
          tempOrder.push(defaultGroup);
        }

        this.groupList = tempOrder;
      });

    this._allCheckedBySearchUser();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit() {
    if (!!this.psDirectiveRef) {
      this.psDirectiveRef.update();
    }
  }

  onCheckForGroup(checkbox: MatCheckbox, group: GroupDetailData) {
    const findGroup = this.selectedGroupList.filter(
      (selectGroup) => selectGroup.seq === group.seq
    );

    if (!!findGroup && findGroup.length > 0 && !checkbox.checked) {
      this.selectedGroupList = this.selectedGroupList.filter(
        (u) => u.seq !== findGroup[0].seq
      );
    } else if (!!findGroup && findGroup.length === 0 && checkbox.checked) {
      this.selectedGroupList = [...this.selectedGroupList, group];
    }

    if (this.selectedGroupList.length === this.groupList.length) {
      this.checkboxAllGroup.checked = true;
    } else {
      this.checkboxAllGroup.checked = false;
    }
    this.changeGroupList.emit({ checked: checkbox.checked, group });
  }

  onToggleCheck(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    this.changeUserList.emit(datas);
  }

  onSearched(users: UserInfoSS[]) {
    this.searchUserInfos = users;
    this._allCheckedBySearchUser();
  }

  onChangedCompanySearch() {
    if (!!this.companySearchData && this.companySearchData.bySearch) {
      this.isSearch = true;
    }
  }

  onCanceled() {
    this.isSearch = false;
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  /** 개별 그룹 체크여부 */
  getCheckedGroup(group: GroupDetailData) {
    if (!!this.selectedGroupList && this.selectedGroupList.length > 0) {
      return (
        this.selectedGroupList.filter((item) => item.seq === group.seq).length >
        0
      );
    }

    return false;
  }

  checkVisible(group: GroupDetailData): boolean {
    if (!!this.curGroup && this.curGroup.seq === group.seq) {
      return false;
    }
    return true;
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.groupName = data.groupName;
    this.changeGroupName.emit(data);
  }

  onAllCheckedGroup(event: MatCheckboxChange) {
    if (event.checked) {
      this.groupList.map((group) => {
        this.selectedGroupList.push(group);
        this.changeGroupList.emit({ checked: true, group });
      });
    } else {
      this.selectedGroupList = [];
      this.groupList.map((group) =>
        this.changeGroupList.emit({ checked: false, group })
      );
    }
  }
  onAllCheckedSearch(event: MatCheckboxChange) {
    if (event.checked) {
      this.groupProfileList.checkAll();
    } else {
      this.groupProfileList.uncheckAll();
    }
  }

  private _allCheckedBySearchUser() {
    if (
      !!this.searchUserInfos &&
      this.searchUserInfos.length > 0 &&
      !!this.selectedUserList &&
      this.selectedUserList.length > 0
    ) {
      const tempList = [];

      this.searchUserInfos.map((searchUser) => {
        this.selectedUserList.every((selectUser) => {
          if (searchUser.seq === selectUser.seq) {
            tempList.push(searchUser);
            return false;
          }
          return true;
        });
      });

      if (tempList.length === this.searchUserInfos.length) {
        this.checkboxAllSearch.checked = true;
      } else {
        this.checkboxAllSearch.checked = false;
      }
    } else if (!!this.checkboxAllSearch) {
      this.checkboxAllSearch.checked = false;
    }
  }
}
