import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  Input,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';
import { MatStepper } from '@angular/material/stepper';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { I18nService } from '@ucap/ng-i18n';
import { GroupActions, GroupSelector } from '@ucap/ng-store-group';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui';
import { takeUntil } from 'rxjs/operators';
import { AppGroupService } from '@app/services/app-group.service';

import { SelectUserSectionComponent } from '../components/select-user.section.component';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

export interface CreateDialogData {
  title: string;
}
export interface CreateDialogResult {}

@Component({
  selector: 'app-dialog-group-create',
  templateUrl: './create.dialog.component.html',
  styleUrls: ['./create.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CreateDialogComponent implements OnInit, OnDestroy {
  @ViewChild('groupSelectUser', { static: false })
  groupSelectUser: SelectUserSectionComponent;

  constructor(
    public dialogRef: MatDialogRef<CreateDialogData, CreateDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: CreateDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private formBuilder: FormBuilder,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private appGroupService: AppGroupService
  ) {}

  private ngOnDestroySubject: Subject<void>;
  private groupName: string;
  private defaultGroup: GroupDetailData;

  currentStep = 0;
  isDisable = true;

  selectedUserList: UserInfoTypes[] = [];

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.defaultGroup = groups.filter((group) => group.seq === 0)[0];
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onCancel(stepper: MatStepper) {
    if (stepper.selectedIndex > 0) {
      this.currentStep--;
      stepper.previous();
      return;
    }
    this.dialogRef.close();
  }
  onConfirm(stepper: MatStepper) {
    let contents: string;
    let isDefaultGroup = false;

    if (this.currentStep === 0) {
      if (
        !this.groupName ||
        this.groupName.localeCompare('') === 0 ||
        this.groupName.trim().localeCompare('') === 0
      ) {
        contents = this.i18nService.t('group:error.requireGroupName');
      } else if (this.isDisable) {
        contents = this.i18nService.t('group:error.invalidGroupName');
      }
    } else {
      if (
        !this.groupName ||
        this.groupName.localeCompare('') === 0 ||
        this.groupName.trim().localeCompare('') === 0
      ) {
        isDefaultGroup = true;
      }
      if (!!this.selectedUserList && this.selectedUserList.length === 0) {
        contents = this.i18nService.t('group:error.notSelectedUser');
      }
    }

    if (!!contents && contents.localeCompare('') !== 0) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('group:error.title.default'),
          html: contents
        }
      });
      return;
    }

    const userSeqs: string[] = [];
    this.selectedUserList.map((user) => userSeqs.push(String(user.seq)));

    if (!isDefaultGroup) {
      this.appGroupService.createGroup(this.groupName, this.selectedUserList);
    } else {
      this.appGroupService.addMemberToGroup(
        this.defaultGroup,
        this.selectedUserList
      );
    }

    this.dialogRef.close();
  }
  onCompleteConfirm(stepper: MatStepper) {
    this.currentStep++;
    stepper.next();
    this.groupSelectUser.psUpdate();
  }

  checkDisableConfirmBtn() {
    if (!!this.groupName && this.groupName.trim().localeCompare('') === 0) {
      return true;
    } else if (
      this.isDisable &&
      !!this.groupName &&
      this.groupName.trim().localeCompare('') !== 0
    ) {
      return true;
    }

    return false;
  }
  onChangeUserList(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => String(u.seq) === String(d.userInfo.seq)
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => String(p.seq) === String(u.seq))
      );
    }
  }

  // onChangeUserList(data: { checked: boolean; userInfo: UserInfoSS }) {
  //   const i = this.selectedUserList.findIndex(
  //     (u) => u.seq === data.userInfo.seq
  //   );

  //   if (data.checked) {
  //     if (-1 === i) {
  //       this.selectedUserList = [...this.selectedUserList, data.userInfo];
  //     }
  //   } else {
  //     if (-1 < i) {
  //       this.selectedUserList = this.selectedUserList.filter(
  //         (u) => u.seq !== data.userInfo.seq
  //       );
  //     }
  //   }
  // }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.isDisable = data.invalid;
    this.groupName = data.groupName;
  }
  onChangeGroupList(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter(
            (user) => String(user.seq) === String(item.seq)
          ).length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter(
            (del) => String(del.seq) === String(item.seq)
          ).length === 0
      );
    }
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
