import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';
import { I18nService } from '@ucap/ng-i18n';

import { SelectUserDialogType, GroupManageType } from '@app/types';

import { UserInfoTypes } from '@app/types';
import { AppService } from '@app/services/app.service';
import { takeUntil } from 'rxjs/operators';
import { GroupSelector } from '@ucap/ng-store-group';
export interface EditUserDialogData {
  title: string;
  type: GroupManageType;
  group?: GroupDetailData;
  userInfos: UserInfoTypes[];
}
export interface EditUserDialogResult {
  type: GroupManageType;
  groupName: string;
  group: GroupDetailData;
  selelctUserList?: UserInfoTypes[];
  selectGroupList?: GroupDetailData[];
}

@Component({
  selector: 'app-dialog-edit-user',
  templateUrl: './edit-user.dialog.component.html',
  styleUrls: ['./edit-user.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class EditUserDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<EditUserDialogData, EditUserDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: EditUserDialogData,
    private appService: AppService,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();

  currentManageType: GroupManageType;
  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupDetailData[] = [];
  groupName = '';
  SelectUserDialogType = SelectUserDialogType;
  groupList: GroupDetailData[] = [];

  ngOnInit(): void {
    this.selectedUserList = [...this.data.userInfos];
    this.currentManageType = this.data.type;

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onConfirm() {
    if (!!this.groupName && this.groupName.trim().localeCompare('') !== 0) {
      this.currentManageType = GroupManageType.Create;
    } else {
      this.groupName = undefined;
    }

    this.dialogRef.close({
      type: this.currentManageType,
      groupName: this.groupName,
      group: this.data.group,
      selelctUserList: this.selectedUserList,
      selectGroupList: this.selectedGroupList
    });
  }

  onChangeSelectedUserList(
    datas: {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  ) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => u.seq === d.userInfo.seq
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
      );
    }
  }

  onChangeGroupList(data: { checked: boolean; group: GroupDetailData }) {
    const i = this.selectedGroupList.findIndex(
      (u) => String(u.seq) === String(data.group.seq)
    );

    if (data.checked) {
      if (-1 === i) {
        this.selectedGroupList = [...this.selectedGroupList, data.group];
      }
    } else {
      if (-1 < i) {
        this.selectedGroupList = this.selectedGroupList.filter(
          (u) => String(u.seq) !== String(data.group.seq)
        );
      }
    }
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.groupName = data.groupName;
  }

  checkDisableBtn(): boolean {
    const errMsg = this.appService.validateGroupName(
      this.groupName,
      this.groupList
    );

    if (
      !!errMsg &&
      this.groupName === '' &&
      !!this.selectedGroupList &&
      this.selectedGroupList.length > 0
    ) {
      return false;
    } else if (
      !!errMsg &&
      !!this.selectedGroupList &&
      this.selectedGroupList.length === 0
    ) {
      return true;
    } else if (!!errMsg && this.groupName !== '') {
      return true;
    }

    return false;
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    // if (userInfo.seq === this.data.userInfo.seq) {
    //   return;
    // }
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
  }

  removableForSelection = (userInfo: UserInfo) => {
    // if (String(userInfo.seq) === String(this.data.userInfo.seq)) {
    //   return false;
    // }
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    // if (String(userInfo.seq) === String(this.data.userInfo.seq)) {
    //   return 'primary';
    // }
    return 'accent';
  };
}
