import { Subject, of } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild,
  ViewContainerRef
} from '@angular/core';

import { Store } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';

import { I18nService } from '@ucap/ng-i18n';

import { GroupManageType } from '@app/types';
import { LogService } from '@ucap/ng-logger';
import { SelectUserSectionComponent } from '../components/select-user.section.component';
import { DefaultDialogLayoutComponent } from '@app/layouts/components/default-dialog.layout.component';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

export interface ManageDialogData {
  title: string;
  groupBuddyList?: { group: GroupDetailData; buddyList: UserInfo[] };
}
export interface ManageDialogResult {
  choice: boolean;
  group?: GroupDetailData;
  selelctUserList?: UserInfoTypes[];
}

@Component({
  selector: 'app-dialog-group-manage',
  templateUrl: './manage.dialog.component.html',
  styleUrls: ['./manage.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ManageDialogComponent implements OnInit, OnDestroy, AfterViewInit {
  constructor(
    public dialogRef: MatDialogRef<ManageDialogData, ManageDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ManageDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private logService: LogService
  ) {}

  @ViewChild('dialogContainer', { static: true, read: ViewContainerRef })
  dialogContainer: ViewContainerRef;

  @ViewChild('appGroupSelectUser', { static: false })
  appGroupSelectUser: SelectUserSectionComponent;

  private ngOnDestroySubject: Subject<void> = new Subject();

  GroupManageType = GroupManageType;

  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupDetailData[] = [];

  ngOnInit(): void {
    this.selectedUserList = this.data.groupBuddyList.buddyList;
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit() {
    this.psUpdate();
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  psUpdate() {
    this.appGroupSelectUser.psUpdate();
  }
  onSelectBySectionGroup(data: { checked: boolean; group: GroupDetailData }) {
    const i = this.selectedGroupList.findIndex((u) => u.seq === data.group.seq);

    if (data.checked) {
      if (-1 === i) {
        this.selectedGroupList = [...this.selectedGroupList, data.group];
      }
    } else {
      if (-1 < i) {
        this.selectedGroupList = this.selectedGroupList.filter(
          (u) => u.seq !== data.group.seq
        );
      }
    }
  }

  onSelectBySectionUserGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter((user) => user.seq === item.seq)
            .length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter((del) => del.seq === item.seq)
            .length === 0
      );
    }
  }

  onChangeSelectedUserList(
    datas: {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  ) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => u.seq === d.userInfo.seq
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
      );
    }
  }
  onCancel(event: MouseEvent) {
    this.dialogRef.close({ choice: false });
  }
  onConfirm(event: MouseEvent) {
    this.dialogRef.close({
      choice: true,
      group: {
        ...this.data.groupBuddyList.group,
        userSeqs: []
      },
      selelctUserList: this.selectedUserList
    });
  }

  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoSS) {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      return (
        this.selectedUserList.filter(
          (item) => String(item.seq) === String(userInfo.seq)
        ).length > 0
      );
    }

    return false;
  }

  checkDisableBtn(): boolean {
    let findUser: UserInfoTypes[];
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      findUser = this.selectedUserList.filter((user) => {
        return this.data.groupBuddyList.group.userSeqs.includes(
          String(user.seq)
        );
      });
    }

    if (!!findUser && findUser.length > 0) {
      return false;
    } else {
      return true;
    }
  }

  onToggleCheck(data: { checked: boolean; userInfo: UserInfoSS }) {
    this.onChangeSelectedUserList([data]);
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const i = this.selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserList = this.selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
