import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { select, Store } from '@ngrx/store';

import { FileUploadItem } from '@ucap/api';
import { FileProfileSaveRequest } from '@ucap/api-common';
import { VersionInfo2Response } from '@ucap/api-public';
import { UserInfoSS, UserInfoF } from '@ucap/protocol-query';
import { LoginResponse } from '@ucap/protocol-authentication';
import { UserInfoUpdateType, User } from '@ucap/protocol-info';

import { LogService } from '@ucap/ng-logger';

import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';

import { UserInfoTypes } from '@app/types';
import { AppGroupService } from '@app/services/app-group.service';
import { AppFileService } from '@app/services/app-file.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppChatService } from '@app/services/app-chat.service';

export interface ProfileDialogData {
  userSeq: string;
}
export interface ProfileDialogResult {}

@Component({
  selector: 'app-dialog-organization-profile',
  templateUrl: './profile.dialog.component.html',
  styleUrls: ['./profile.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileDialogComponent implements OnInit, OnDestroy {
  constructor(
    private store: Store<any>,
    public dialogRef: MatDialogRef<ProfileDialogData, ProfileDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ProfileDialogData,
    private logService: LogService,
    private changeDetectorRef: ChangeDetectorRef,
    public matDialog: MatDialog,
    private appAuthenticationService: AppAuthenticationService,
    private appGroupService: AppGroupService,
    private appChatService: AppChatService,
    private appFileServie: AppFileService
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();
  private loginRes: LoginResponse;
  private user: User;
  private versionInfo2Res: VersionInfo2Response;

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginRes))
      .subscribe((loginRes) => {
        this.loginRes = loginRes;
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onOpenCaht(userInfo: UserInfoSS) {
    this.appChatService.newOpenRoom(
      [Number(userInfo.seq) as any],
      false,
      this.user
    );
    this.dialogRef.close();
  }
  onSendMessage(userInfo: UserInfoSS) {}
  onSendCall(call: string) {}
  onSendSms(employeeNum: string) {}
  onCreateConference(userSeq: number) {}
  onToggleFavorit(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.appGroupService.updateBuddy(params.userInfo, params.isFavorite);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    this.appGroupService
      .updateBuddyByToggle(params)
      .then((isRemoveBuddy) => {
        this.changeDetectorRef.markForCheck();
      })
      .catch((reson) => this.logService.error(reson));
  }
  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    const loginSession = this.appAuthenticationService.getLoginSession();

    const profile = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      token: this.loginRes.tokenString,
      file: profileImageFileUploadItem.file,
      fileUploadItem: profileImageFileUploadItem
    } as FileProfileSaveRequest;

    this.appFileServie.fileProfileSave(
      profile,
      this.versionInfo2Res.profileUploadUrl
    );
  }
  onUpdateIntro(intro: string) {
    this.appGroupService.updateIntro(intro, UserInfoUpdateType.Intro);
  }
  onUpdateNickname(params: { userInfo: UserInfoTypes; nickname: string }) {
    this.appGroupService.updateNickname(
      params.userInfo as UserInfoF,
      params.nickname
    );
  }
}
