import { of } from 'rxjs';
import { take, catchError, map } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { MatDialog } from '@angular/material/dialog';

import { Store } from '@ngrx/store';

import { NativeService } from '@ucap/native';
import { GroupDetailData, UserInfo } from '@ucap/protocol-sync';
import { UserInfoF, UserInfoSS, UserInfoDN } from '@ucap/protocol-query';
import { UserInfoUpdateType } from '@ucap/protocol-info';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { UserActions } from '@ucap/ng-store-organization';
import { LoginActions } from '@ucap/ng-store-authentication';
import { GroupActions, BuddyActions } from '@ucap/ng-store-group';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui';

import { EditInlineInputDialogComponent } from '@app/sections/group/dialogs/edit-inline-input.dialog.component';
import { UserInfoTypes, GroupManageType } from '@app/types';
import {
  EditUserDialogComponent,
  EditUserDialogData,
  EditUserDialogResult
} from '@app/sections/group/dialogs/edit-user.dialog.component';

@Injectable({
  providedIn: 'root'
})
export class AppGroupService {
  constructor(
    private dialog: MatDialog,
    private store: Store<any>,
    private i18nService: I18nService,
    protected logService: LogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  addMemberToGroup(
    targetGroup: GroupDetailData,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [...targetGroup.userSeqs];

    userInfoList.map((userInfo) => {
      const find = targetUserSeqs.indexOf(String(userInfo.seq));
      if (find < 0) {
        targetUserSeqs.push(String(userInfo.seq));
      }
    });

    this._updateMember(targetGroup, targetUserSeqs);
  }

  moveMemberToGroup(
    fromGroup: GroupDetailData,
    toGroup: GroupDetailData,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [];

    userInfoList.map((userInfo) => {
      targetUserSeqs.push(String(userInfo.seq));
    });

    this.store.dispatch(
      GroupActions.moveMember({
        fromGroup,
        toGroup,
        targetUserSeq: targetUserSeqs
      })
    );
  }
  copyMemberToGroup(
    targetGroup: GroupDetailData,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [...targetGroup.userSeqs];

    userInfoList.map((userInfo) => {
      const find = targetUserSeqs.indexOf(String(userInfo.seq));
      if (find < 0) {
        targetUserSeqs.push(String(userInfo.seq));
      }
    });

    this._updateMember(targetGroup, targetUserSeqs);
  }
  removeMemberToGroup(
    contens: string,
    userInfoList: UserInfoTypes[],
    group: GroupDetailData
  ): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeBuddy'),
            html: contens
          }
        });
        dialogRef
          .afterClosed()
          .pipe(
            take(1),
            map((result) => {
              if (!!result && result.choice) {
                const trgtUserSeq: string[] = [];
                const selectSeq: string[] = [];

                userInfoList.map((user) => selectSeq.push(String(user.seq)));

                group.userSeqs.filter((buddySeq) => {
                  if (
                    !trgtUserSeq.includes(buddySeq) &&
                    !selectSeq.includes(buddySeq)
                  ) {
                    trgtUserSeq.push(buddySeq);
                  }
                });

                this._updateMember(group, trgtUserSeq);
                resolve();
              }
            }),
            catchError((err) => {
              return of(err);
            })
          )
          .subscribe();
      } catch (err) {
        rejects();
      }
    });
  }

  createGroup(groupName: string, userInfoList: UserInfoTypes[]): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const targetUserSeqs: string[] = [];
        userInfoList.map((user) => targetUserSeqs.push(String(user.seq)));
        this.store.dispatch(
          GroupActions.create({
            groupName,
            targetUserSeqs
          })
        );
        resolve();
      } catch (error) {
        rejects(error);
      }
    });
  }
  removeGroup(groupBuddyList: {
    group: GroupDetailData;
    buddyList: UserInfo[];
  }): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeGroup'),
            html: this.i18nService.t('group:dialog.removeGroupConfirm', {
              targetGroup: `${groupBuddyList.group.name}`
            })
          }
        });
        dialogRef
          .afterClosed()
          .pipe(take(1))
          .subscribe((result) => {
            if (!!result && !!result.choice) {
              this.store.dispatch(
                GroupActions.del({ group: groupBuddyList.group })
              );

              resolve();
            }
          });
      } catch (error) {
        rejects(error);
      }
    });
  }
  updateGroupName(group: GroupDetailData, rect: any): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const dialogRef = this.dialog.open(EditInlineInputDialogComponent, {
          width: rect.width,
          height: rect.height,
          panelClass: 'ucap-edit-group-name-dialog',
          data: {
            type: 'GROUP_NAME',
            curValue: group.name,
            placeholder: this.i18nService.t('group:dialog.newGroupName'),
            maxLength: 20,
            left: rect.left,
            top: rect.top
          }
        });

        dialogRef
          .afterClosed()
          .pipe(
            take(1),
            map((result) => {
              if (
                !!result &&
                result.choice &&
                result.curValue.localeCompare(name) !== 0
              ) {
                this.store.dispatch(
                  GroupActions.update({
                    req: {
                      groupSeq: group.seq,
                      groupName: result.curValue,
                      userSeqs: group.userSeqs
                    }
                  })
                );
                resolve();
              }
            }),
            catchError((err) => {
              return of(err);
            })
          )
          .subscribe();
      } catch (error) {
        rejects(error);
      }
    });
  }

  updateBuddyByToggle(params: {
    userInfo: UserInfoSS;
    isBuddy: boolean;
  }): Promise<boolean> {
    return new Promise<boolean>((resolve, rejects) => {
      try {
        if (params.isBuddy) {
          // 동료추가
          const dialogRef = this.dialog.open<
            EditUserDialogComponent,
            EditUserDialogData,
            EditUserDialogResult
          >(EditUserDialogComponent, {
            width: '100%',
            height: '100%',
            data: {
              title: this.i18nService.t('group:dialog.title.addBuddy'),
              type: GroupManageType.Add,
              userInfos: [params.userInfo]
            }
          });
          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result: EditUserDialogResult) => {
                if (result.type === GroupManageType.Add) {
                  if (
                    !!result.selectGroupList &&
                    result.selectGroupList.length > 0
                  ) {
                    result.selectGroupList.forEach((g) => {
                      this.addMemberToGroup(g, result.selelctUserList);
                    });
                  }
                } else if (result.type === GroupManageType.Create) {
                  this.createGroup(result.groupName, result.selelctUserList);
                }
                resolve(false);
              }),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        } else {
          // 동료삭제

          this.removeBuddy(params.userInfo)
            .then(() => {
              resolve(true);
            })
            .catch((reason) => {})
            .finally(() => {});
        }
      } catch (error) {
        rejects(error);
      }
    });
  }

  updateBuddy(userInfo: UserInfoTypes, isFavorite: boolean): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        this.store.dispatch(
          BuddyActions.update({
            req: { seq: Number(userInfo.seq), isFavorit: isFavorite }
          })
        );
        resolve();
      } catch (error) {
        rejects(error);
      }
    });
  }
  removeBuddy(userInfo: UserInfoTypes): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const tempUser = userInfo as
          | UserInfo
          | UserInfoSS
          | UserInfoF
          | UserInfoDN;

        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeBuddy'),
            html: this.i18nService.t(
              'group:dialog.removeBuddyFromProfile',

              { target: `${tempUser.name} ${tempUser.grade}` }
            )
          }
        });

        dialogRef
          .afterClosed()
          .pipe(take(1))
          .subscribe((result) => {
            if (!!result && !!result.choice) {
              this.store.dispatch(
                BuddyActions.delAndClear({ seq: Number(userInfo.seq) })
              );

              resolve();
            }
          });
      } catch (error) {
        rejects(error);
      }
    });
  }
  updateNickname(
    userInfo: UserInfoF,
    nickname: string,
    rect?: any
  ): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        if (!!rect) {
          const dialogRef = this.dialog.open(EditInlineInputDialogComponent, {
            width: rect.width - 30 + '',
            height: rect.height,
            panelClass: 'ucap-edit-group-name-dialog',
            data: {
              type: 'NICKNAME',
              curValue: userInfo.nickName,
              placeholder: this.i18nService.t('group:dialog.newNickname'),
              maxLength: 20,
              left: rect.left + 70,
              top: rect.top
            }
          });

          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result) => {
                if (
                  !!result &&
                  result.choice &&
                  result.curValue.localeCompare(userInfo.nickName) !== 0
                ) {
                  this._updateNickname(userInfo.seq, result.curValue);
                  resolve();
                }
              }),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        } else {
          if (nickname.trim().localeCompare('') === 0) {
            nickname = ' ';
          }
          this._updateNickname(userInfo.seq, nickname);
          resolve();
        }
      } catch (error) {
        rejects(error);
      }
    });
  }

  updateIntro(info: string, type: UserInfoUpdateType) {
    if (!!info && info.trim().localeCompare('') === 0) {
      info = ' ';
    }
    this.store.dispatch(
      UserActions.modifyInfo({
        req: {
          type,
          info
        }
      })
    );
  }
  private _updateNickname(userSeq: number | string, nickname: string) {
    let userSeqf: number;
    try {
      userSeqf = Number(userSeq);
    } catch (error) {
      this.logService.error(error);
    }

    this.store.dispatch(
      BuddyActions.nickname({
        req: {
          userSeq: Number(userSeq),
          nickname
        }
      })
    );
  }

  private _updateMember(
    targetGroup: GroupDetailData,
    targetUserSeqs: string[]
  ) {
    this.store.dispatch(
      GroupActions.updateMember({
        targetGroup,
        targetUserSeqs
      })
    );
  }
}
