import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  Input,
  EventEmitter,
  Output,
  AfterViewInit,
  ChangeDetectorRef,
  OnDestroy,
  Inject
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LoginSession, DeviceType } from '@ucap/core';
import { NativeService } from '@ucap/native';
import { FileDownloadItem } from '@ucap/api';
import { LoginResponse } from '@ucap/protocol-authentication';
import {
  Info,
  EventJson,
  EventType,
  FileType,
  StickerEventJson,
  isRecalled,
  isCopyable,
  isRecallable,
  isForwardable,
  FileEventJson,
  MassTranslationEventJson
} from '@ucap/protocol-event';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo,
  RoomType
} from '@ucap/protocol-room';
import { User } from '@ucap/protocol-info';

import { I18nService } from '@ucap/ng-i18n';
import { LocalStorageService } from '@ucap/ng-web-storage';
import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { UserSelector } from '@ucap/ng-store-organization';
import { LoginSelector } from '@ucap/ng-store-authentication';

import { SelectFileInfo } from '@ucap/ng-ui';

import { AppAuthenticationService } from '@app/services/app-authentication.service';

import {
  TextDetailDialogComponent,
  TextDetailDialogData,
  TextDetailDialogResult
} from '@app/sections/chat/dialogs/text-detail.dialog.component';
import { AppFileService } from '@app/services/app-file.service';

@Component({
  selector: 'app-chat-message-box',
  templateUrl: './message-box.component.html',
  styleUrls: ['./message-box.component.scss']
})
export class MessageBoxComponent implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  message: Info<EventJson>;
  @Input()
  roomInfo: RoomInfo;

  @Input()
  isMe = false;
  @Input()
  translationSimpleview: boolean;
  @Input()
  senderInfo: RoomUserInfoShort | RoomUserInfo;
  @Input()
  defaultProfileImage = '';
  @Input()
  profileImageRoot = '';

  @Input()
  dateChanged = false;
  @Input()
  unreadCount: number;

  loginSession: LoginSession;
  loginRes: LoginResponse;
  user: User;

  EventType = EventType;

  isRecalledMessage = isRecalled;
  isCopyableMessage = isCopyable;
  isRecallableMessage = isRecallable;
  isForwardableMessage = isForwardable;

  //////////////////////////////////
  @Input()
  messageType: string;
  @Input()
  highlight = false;
  @Input()
  existReadToHere = false;
  @Input()
  fileType: FileType;

  @Output()
  fileViewer: EventEmitter<FileEventJson> = new EventEmitter();

  @Output()
  messageContextMenu = new EventEmitter<{
    menuType: string;
    message: Info<EventJson>;
  }>();

  @Output()
  massTranslationDetail = new EventEmitter<{
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }>();

  @Output()
  openProfile = new EventEmitter<string>();

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private i18nService: I18nService,
    private dialog: MatDialog,
    private localStorageService: LocalStorageService,
    private logService: LogService,
    private appAuthenticationService: AppAuthenticationService,
    private appFileService: AppFileService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginRes))
      .subscribe((loginRes) => {
        this.loginRes = loginRes;
      });

    this.loginSession = this.appAuthenticationService.getLoginSession();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {}

  /**
   * 정보성 Event 인지 판단.
   * @description 정보성 event 일 경우 프로필, 일시 를 표현하지 않는다.
   * Edit with reducers.ts / sync / updateRoomForNewEventMessage
   */
  isInformation(info: Info<EventJson>) {
    if (
      info.type === EventType.Join ||
      info.type === EventType.Exit ||
      info.type === EventType.ForcedExit ||
      info.type === EventType.RenameRoom ||
      info.type === EventType.NotificationForiOSCapture ||
      info.type === EventType.NotificationForTimerRoom ||
      info.type === EventType.GuideForRoomTimerChanged
    ) {
      return true;
    }
    return false;
  }

  /**
   * Get Sticker image.
   */
  getStickerImage(message: Info<EventJson>): string {
    if (!!message.sentMessageJson) {
      return `assets/sticker/sticker_s_${
        (message.sentMessageJson as StickerEventJson).file
      }.png`;
    } else {
      return '';
    }
  }

  onOpenProfile(userInfo: RoomUserInfoShort | RoomUserInfo): void {
    this.openProfile.emit(String(userInfo.seq));
  }

  /**
   * Url link Open Event.
   */
  onOpenLink(url: string): void {
    this.nativeService.platform_openDefaultBrowser(url);
  }

  /**
   * Detail view > Mass text.
   */
  onOpenMassDetail(eventMassSeq: number): void {
    const dialogRef = this.dialog.open<
      TextDetailDialogComponent,
      TextDetailDialogData,
      TextDetailDialogResult
    >(TextDetailDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        message: this.message,
        roomId: this.roomInfo.roomId,
        userSeq: String(this.user.info.seq),
        deviceType: this.loginSession.deviceType,
        token: this.loginRes.tokenString,
        eventMassSeq: Number(eventMassSeq)
      }
    });
  }

  /**
   * Detail view > Mass translation text
   */
  onOpenMassTranslationDetail(params: {
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }) {
    this.massTranslationDetail.emit(params);
  }
  isShowUnreadCount = () => {
    if (
      !!this.roomInfo &&
      [RoomType.Mytalk, RoomType.Allim, RoomType.Bot, RoomType.Link].some(
        (v) => v === this.roomInfo.roomType
      )
    ) {
      return false;
    }
    return true;
  };

  onClickMessageContextMenu(menuType: string) {
    this.messageContextMenu.emit({
      menuType,
      message: this.message
    });
  }

  /** [Event] Image Viewer */
  openViewer(fileEvent: SelectFileInfo): void {
    this.fileViewer.emit(fileEvent);
  }

  fileSave(params: {
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }): void {
    if (
      params.type === 'saveAs' &&
      this.loginSession.deviceType === DeviceType.PC
    ) {
      this.nativeService
        .file_selectForSave({ defaultPath: params.fileInfo.fileName })
        .then((result) => {
          if (!!result) {
            if (!!result.canceled) {
              // 취소함.
            } else {
              this.appFileService.saveFile(
                {
                  fileInfo: params.fileInfo,
                  fileDownloadItem: params.fileDownloadItem,
                  type: params.type,
                  fileName: params.fileInfo.fileName,
                  fileDownloadUrl: undefined,
                  savePath: result.filePath
                },
                this.loginRes,
                this.user,
                this.loginSession
              );
            }
          }
        })
        .catch((reason) => {
          // this.snackBarService.open(
          //   this.translateService.instant(
          //     'common:file.errors.failToSpecifyPath'
          //   ),
          //   this.translateService.instant('common:file.errors.label')
          // );
        });
    } else {
      this.appFileService.saveFile(
        {
          fileInfo: params.fileInfo,
          fileDownloadItem: params.fileDownloadItem,
          type: params.type,
          fileName: params.fileInfo.fileName,
          fileDownloadUrl: undefined,
          savePath: undefined
        },
        this.loginRes,
        this.user,
        this.loginSession
      );
    }
  }
}
