import { Subject, combineLatest, of } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  AfterViewInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ViewChild,
  ElementRef,
  Self
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatMenuTrigger } from '@angular/material/menu';

import { StatusCode, PresenceType } from '@ucap/core';
import { VersionInfo2Response } from '@ucap/api-public';
import { StatusBulkInfo } from '@ucap/protocol-status';
import { UserInfo, GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';
import { User } from '@ucap/protocol-info';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { BuddySelector, GroupSelector } from '@ucap/ng-store-group';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import {
  ExpansionComponent as UcapExpansionComponent,
  NodeType,
  FlatNode,
  GroupNode
} from '@ucap/ng-ui-group';
import { PresenceUtil } from '@ucap/ng-ui-organization';

import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { LoginSession } from '@app/models/login-session';

import { environment } from '@environments';
import { GroupOpenInfo } from '@app/models/group-open-info';
import { SortViewType } from '@app/pages/group/types/sort-view.type';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

@Component({
  selector: 'app-group-expansion-01',
  templateUrl: './expansion-01.component.html',
  styleUrls: ['./expansion-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Expansion01Component implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  selectedUserList: UserInfoTypes[];

  @Input()
  showType: SortViewType;

  @Input()
  loginSession: LoginSession;

  @Output()
  profileMenu: EventEmitter<any> = new EventEmitter();

  @Output()
  clicked = new EventEmitter<{ event: MouseEvent; userInfo: UserInfoTypes }>();

  @Output()
  selectGroupMenu = new EventEmitter<{
    menuType: string;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
    rect: any;
  }>();

  @Output()
  selectProfileMenu: EventEmitter<{
    menuType: string;
    userInfo: UserInfoF;
    group: GroupDetailData;
    rect: any;
  }> = new EventEmitter();

  @Output()
  toggleCheckUser: EventEmitter<
    {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  > = new EventEmitter();

  @Output()
  toggleCheckGroup = new EventEmitter<{
    isChecked: boolean;
    groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] };
  }>();

  @Output()
  floatingProfileMenu = new EventEmitter<{
    menuType: string;
    userInfo: UserInfoF;
  }>();

  @Output()
  groupDestroy = new EventEmitter<GroupOpenInfo>();

  @ViewChild('groupExpansion', { static: false })
  groupExpansion: UcapExpansionComponent;

  @ViewChild('groupContextMenuTrigger', { static: true })
  groupContextMenuTrigger: MatMenuTrigger;
  groupContextMenuPosition = { x: '0px', y: '0px' };

  @ViewChild('editGroupNameTrigger', { static: true })
  editGroupNameTrigger: MatMenuTrigger;
  editGroupNamePosition = { x: '0px', y: '0px' };

  @ViewChild('profileContextMenuTrigger', { static: true })
  profileContextMenuTrigger: MatMenuTrigger;
  profileContextMenuPosition = { x: '0px', y: '0px' };

  versionInfo2Res: VersionInfo2Response;
  user: User;

  groupMenuEvent: MouseEvent;
  statusBulkInfos: StatusBulkInfo[];
  currentSelectUser: UserInfo;

  displayOrder: NodeType[] = [
    NodeType.Profile,
    NodeType.Favorite,
    NodeType.Buddy,
    NodeType.Default
  ];

  favorites: UserInfo[];
  onlineFavorites: UserInfo[];

  groupBuddies: { group: GroupDetailData; buddyList: UserInfo[] }[];
  onlineBuddies: { group: GroupDetailData; buddyList: UserInfo[] }[];
  onOffBuddies: { group: GroupDetailData; buddyList: UserInfo[] }[];

  groupOpenInfo: GroupOpenInfo;

  editablGroup: GroupDetailData = null;

  String = String;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private appAuthenticationService: AppAuthenticationService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private i18nService: I18nService,
    @Self() private elementRef: ElementRef
  ) {}

  ngOnInit(): void {
    this.groupOpenInfo = this.loginSession.groupInfo;

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    combineLatest([
      this.store.pipe(select(BuddySelector.buddies)),
      this.store.pipe(select(GroupSelector.groups)),
      this.store.pipe(select(PresenceSelector.selectAllStatusBulkInfo))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([buddies, groups, bulkInfos]) => {
        buddies = buddies || [];
        groups = groups || [];
        bulkInfos = bulkInfos || [];

        const favorites = buddies
          .filter((buddy) => buddy.isFavorit)
          .sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));

        if (!!favorites && 0 < favorites.length) {
          this.favorites = favorites;
          this.onlineFavorites = this.favorites.filter((favorite) => {
            const tempBulkInfo = bulkInfos.filter(
              (bulkInfo) => String(bulkInfo.userSeq) === String(favorite.seq)
            );

            if (!!tempBulkInfo && tempBulkInfo.length > 0) {
              const pcStatus = tempBulkInfo[0].pcStatus;
              if (pcStatus === StatusCode.OnLine) {
                return favorite;
              }
            }
          });
          this.changeDetectorRef.markForCheck();
        } else if (!!favorites && favorites.length === 0) {
          this.favorites = undefined;
          this.onlineFavorites = undefined;
          this.changeDetectorRef.markForCheck();
        }

        const tempOrder: GroupDetailData[] = [];
        let defaultGroup: GroupDetailData;
        const buddyGroup: GroupDetailData[] = [];
        groups.forEach((group) => {
          if (0 === group.seq) {
            defaultGroup = group;
          } else {
            buddyGroup.push(group);
          }
        });

        tempOrder.push(
          ...buddyGroup.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          )
        );

        if (!!defaultGroup) {
          tempOrder.push(defaultGroup);
        }

        groups = tempOrder;

        if (!!groups && 0 < groups.length) {
          this.groupBuddies = [];

          for (const group of groups) {
            this.groupBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                return -1 < group.userSeqs.indexOf(String(buddy.seq));
              })
            });
          }

          this.changeDetectorRef.markForCheck();
        }

        if (!!bulkInfos && bulkInfos.length > 0) {
          // 접속한 동료 리스트
          this.onlineBuddies = [];

          for (const group of groups) {
            const tempBuddyList = [];

            this.onlineBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                if (-1 < group.userSeqs.indexOf(String(buddy.seq))) {
                  const tempBulkinfos = bulkInfos.filter(
                    (bulk) => bulk.userSeq === buddy.seq + ''
                  );
                  if (!!tempBulkinfos && tempBulkinfos.length > 0) {
                    const pcStatus = tempBulkinfos[0].pcStatus;
                    const mStatus = tempBulkinfos[0].mobileStatus;

                    if (pcStatus === StatusCode.OnLine) {
                      return tempBuddyList.push(buddy);
                    }
                  }
                }
              })
            });
          }
        }
        // 온/오프라인 리스트
        if (!!bulkInfos && bulkInfos.length > 0) {
          this.onOffBuddies = [];
          const onlineGroup = {
            SENDER_SEQ: String(this.user.info.seq),
            SSVC_TYPE: 5,
            SVC_TYPE: 82,
            seq: -998,
            name: this.i18nService.t('group:label.online'),
            isActive: true
          } as GroupDetailData;
          const offlineGroup = {
            SENDER_SEQ: String(this.user.info.seq),
            SSVC_TYPE: 5,
            SVC_TYPE: 82,
            seq: -999,
            name: this.i18nService.t('group:label.offline'),
            isActive: true
          } as GroupDetailData;

          const tempGroups = [onlineGroup, offlineGroup];

          for (const group of tempGroups) {
            const tempBuddyList = [];
            this.onOffBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                const tempBulkinfos = bulkInfos.filter(
                  (bulk) => bulk.userSeq + '' === buddy.seq + ''
                );
                if (!!tempBulkinfos && tempBulkinfos.length > 0) {
                  const pcStatus = tempBulkinfos[0].pcStatus;
                  const mStatus = tempBulkinfos[0].mobileStatus;

                  if (
                    group.name.localeCompare(
                      this.i18nService.t('group:label.online')
                    ) === 0
                  ) {
                    if (pcStatus === StatusCode.OnLine) {
                      return tempBuddyList.push(buddy);
                    }
                  } else if (
                    group.name.localeCompare(
                      this.i18nService.t('group:label.offline')
                    ) === 0
                  ) {
                    if (
                      pcStatus === StatusCode.Offline &&
                      (mStatus === StatusCode.Offline ||
                        mStatus === StatusCode.OnLine)
                    ) {
                      return tempBuddyList.push(buddy);
                    }
                  }
                }
              })
            });
          }
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectAllStatusBulkInfo)
      )
      .subscribe((statusArr) => {
        this.statusBulkInfos = statusArr;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.groupOpenInfo && !!this.groupDestroy) {
      this.groupDestroy.emit(this.groupOpenInfo);
    }
  }

  ngAfterViewInit() {
    this._groupOpenBySessionInfo();
  }

  onClickNode(params: { node: FlatNode; isExpand }) {
    // const findSeq = this.groupOpenInfo.groupSeqs.filter(seq => node.node.groupDetail.seq === seq)[0];
    const group = params.node.node.groupDetail;

    const i = this.groupOpenInfo.groupSeqs.findIndex(
      (seq) => seq === group.seq
    );
    if (!!params.isExpand) {
      if (-1 === i) {
        this.groupOpenInfo.groupSeqs.push(group.seq);
      }
    } else {
      if (-1 < i) {
        this.groupOpenInfo.groupSeqs = this.groupOpenInfo.groupSeqs.filter(
          (seq) => seq !== group.seq
        );
      }
    }

    if (
      this.groupExpansion &&
      !!this.groupOpenInfo &&
      this.groupOpenInfo.groupSeqs.length > 0
    ) {
      this.groupExpansion.groupOpenInfos = this.groupOpenInfo.groupSeqs;
    }
  }

  onProfileMenu(
    event: MouseEvent,
    userInfo: UserInfoTypes,
    group: GroupDetailData,
    isSearchData: boolean
  ) {
    this.profileMenu.emit({ event, userInfo, group, isSearchData });
  }

  onClickUser(event: MouseEvent, userInfo: UserInfo) {
    this.currentSelectUser = userInfo;
    this.clicked.emit({ event, userInfo });
  }

  getNodeGroupStatus(groupNode: GroupNode): string {
    if (
      !!groupNode &&
      !!this.showType &&
      this.showType === SortViewType.onOff
    ) {
      return `${groupNode.children.length}`;
    }

    if (
      !!this.statusBulkInfos &&
      this.statusBulkInfos.length > 0 &&
      !!groupNode &&
      !!groupNode.groupDetail &&
      !!groupNode.groupDetail.userSeqs &&
      groupNode.groupDetail.userSeqs.length > 0 &&
      !!this.showType &&
      this.showType !== SortViewType.onOff
    ) {
      let onlineCount = 0;
      this.statusBulkInfos.map((statusInfo) => {
        groupNode.groupDetail.userSeqs.filter((seq) => {
          if (Number(seq) === Number(statusInfo.userSeq)) {
            const pcOnline = PresenceUtil.isOnline(statusInfo, PresenceType.PC);
            const mobileOnline = PresenceUtil.isOnline(
              statusInfo,
              PresenceType.MOBILE
            );

            if (pcOnline) {
              onlineCount++;
            }
          }
        });
      });

      return `(${onlineCount}/${groupNode.children.length})`;
    } else {
      return `(0/0)`;
    }
  }

  onClickMoreMenu(params: { event: MouseEvent; node: FlatNode }) {
    if (this.showType === SortViewType.onOff) {
      return;
    }

    params.event.preventDefault();
    params.event.stopPropagation();
    this.groupMenuEvent = params.event;
    this.groupContextMenuPosition.x = params.event.clientX + 'px';
    this.groupContextMenuPosition.y = params.event.clientY + 'px';
    this.groupContextMenuTrigger.menu.focusFirstItem('mouse');
    this.groupContextMenuTrigger.menuData = {
      group: params.node.node.groupDetail
    };
    this.groupContextMenuTrigger.openMenu();
  }

  onShowEditGroup(group: GroupDetailData) {
    const x = this.groupMenuEvent.clientX - 20;
    const y = this.groupMenuEvent.clientY - 30;

    this.editGroupNamePosition.x = x + 'px';
    this.editGroupNamePosition.y = y + 'px';
    this.editGroupNameTrigger.menu.focusFirstItem('mouse');
    this.editGroupNameTrigger.menuData = {
      group
    };
    this.editGroupNameTrigger.openMenu();
  }

  onExpandMore() {
    this.groupExpansion.expandMore();
  }
  onExpandLess() {
    this.groupExpansion.expandLess();
  }

  getShowGroupContextMenu(menuType: string, group: GroupDetailData) {
    // 즐겨찾기 그룹 숨김메뉴
    if (
      menuType === 'DIV1' ||
      menuType === 'RENAME' ||
      menuType === 'MANAGE_MEMBER' ||
      menuType === 'COPY_MEMBER' ||
      menuType === 'MOVE_MEMBER' ||
      menuType === 'DELETE'
    ) {
      if (
        !group ||
        group === undefined ||
        (group.seq < 0 && group.name === 'Favorite')
      ) {
        return false;
      }

      /** 수정불가 그룹 핸들링. */
      if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
        const fixedGroupSeqs: number[] =
          environment.productConfig.group.fixedGroupSeqs;
        if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
          if (fixedGroupSeqs.indexOf(group.seq) > -1) {
            return false;
          }
        }
      }
    }

    // 기본 그룹 숨김메뉴
    if (menuType === 'RENAME' || menuType === 'DELETE') {
      if (!!group && group.seq === 0) {
        return false;
      }
    }

    // 그룹원 0명인 그룹 메뉴 정리
    if (
      menuType === 'CHAT' ||
      menuType === 'SEND_MESSAGE' ||
      menuType === 'DIV1' ||
      menuType === 'COPY_MEMBER' ||
      menuType === 'MOVE_MEMBER'
    ) {
      if (!!group && !!group.userSeqs && group.userSeqs.length > 0) {
        return true;
      } else {
        return false;
      }
    }

    return true;
  }

  onSelectGroupMenu(menuType: string, group: GroupDetailData) {
    let rect: any;

    if (menuType.localeCompare('RENAME') === 0) {
      const target = this.elementRef.nativeElement.querySelector(
        '.mat-tree-node'
      );
      const clickEventY = this.groupMenuEvent.clientY;
      const clientRect = target.getBoundingClientRect();

      rect = {
        width: clientRect.width,
        height: clientRect.height,
        top: clickEventY - 100 + clientRect.height,
        left: clientRect.left,
        bottom: clientRect.bottom,
        right: clientRect.right
      };
    }

    let groupBuddyList: { group: GroupDetailData; buddyList: UserInfo[] }[];

    if (group.seq === -9999) {
      groupBuddyList = [];
      groupBuddyList.push({ group, buddyList: this.favorites });
    } else {
      groupBuddyList = this.groupBuddies.filter(
        (g) => g.group.seq === group.seq
      );
    }

    this.selectGroupMenu.emit({
      menuType,
      groupBuddyList: groupBuddyList[0],
      rect
    });
  }

  getFavoritesForShowType(): UserInfo[] {
    if (!!this.showType && this.showType === SortViewType.all) {
      return this.favorites;
    } else if (!!this.showType && this.showType === SortViewType.onlineBuddy) {
      return this.onlineFavorites;
    } else if (!!this.showType && this.showType === SortViewType.onOff) {
      return null;
    }
    return null;
  }

  getBuddiesForShowType(): { group: GroupDetailData; buddyList: UserInfo[] }[] {
    if (!!this.showType && this.showType === SortViewType.all) {
      return this.groupBuddies;
    } else if (!!this.showType && this.showType === SortViewType.onlineBuddy) {
      return this.onlineBuddies;
    } else if (!!this.showType && this.showType === SortViewType.onOff) {
      return this.onOffBuddies;
    }
    return null;
  }

  onClikeMoreProfile(params: {
    event: MouseEvent;
    userInfo: UserInfoTypes;
    group: GroupDetailData;
    rect: any;
  }) {
    params.event.preventDefault();
    params.event.stopPropagation();

    this.profileContextMenuPosition.x = params.event.clientX + 'px';
    this.profileContextMenuPosition.y = params.event.clientY + 'px';
    this.profileContextMenuTrigger.menu.focusFirstItem('mouse');
    this.profileContextMenuTrigger.menuData = {
      userInfo: params.userInfo,
      group: params.group,
      rect: params.rect
    };
    this.profileContextMenuTrigger.openMenu();
  }

  onFloatingProfileMenu(params: { menuType: string; userInfo: UserInfoF }) {
    this.floatingProfileMenu.emit(params);
  }

  onClickProfileContextMenu(
    event: MouseEvent,
    menuType: string,
    userInfo: UserInfoF,
    group: GroupDetailData,
    rect: any
  ) {
    const clientRect = {
      width: rect.width,
      height: rect.height,
      top: rect.top,
      left: rect.left,
      right: rect.right,
      bottom: rect.bottom
    };

    if (menuType === 'VIEW_PROFILE') {
      this.onClickUser(event, userInfo as UserInfo);
    }

    this.selectProfileMenu.emit({
      menuType,
      userInfo,
      group,
      rect: clientRect
    });

    this.profileContextMenuTrigger.closeMenu();
  }

  getGroupNameBySeq(group: GroupDetailData): string {
    if (!!group && group.seq === 0) {
      return this.i18nService.t('group:category.default');
    } else if (!!group && group.seq === -9999) {
      return this.i18nService.t('group:category.favorite');
    }

    return '';
  }
  onProfileMenuClose(event: MouseEvent) {}

  getShowProfileContextMenu(menuType: string, group: GroupDetailData) {
    if (!group || group === undefined) {
      if (
        menuType === 'REGISTER_FAVORITE' ||
        menuType === 'SEND_MESSAGE' ||
        menuType === 'REGISTER_NICKNAME'
      ) {
        // continue;
      } else {
        return false;
      }
    }
    /** 수정불가 그룹 핸들링. */
    if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
      const fixedGroupSeqs: number[] =
        environment.productConfig.group.fixedGroupSeqs;
      if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
        if (fixedGroupSeqs.indexOf(group.seq) > -1) {
          if (menuType === 'REMOVE_FROM_GROUP' || menuType === 'MOVE_BUDDY') {
            return false;
          }
        }
      }
    }

    return true;
  }

  private _groupOpenBySessionInfo() {
    if (
      !!this.loginSession &&
      !!this.groupBuddies &&
      !!this.groupExpansion &&
      this.loginSession.groupInfo.groupSeqs.length > 0 &&
      this.groupBuddies.length > 0
    ) {
      this.groupExpansion.expand(
        this.loginSession.groupInfo.groupSeqs,
        this.loginSession.groupInfo.lastGroupSeq
      );
    }
  }
}
