import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ElementRef,
  Self
} from '@angular/core';

import { select, Store } from '@ngrx/store';

import { VersionInfo2Response } from '@ucap/api-public';
import { GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoF, AuthResponse } from '@ucap/protocol-query';
import { StatusBulkInfo } from '@ucap/protocol-status';

import { I18nService } from '@ucap/ng-i18n';

import {
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import { ucapAnimations } from '@ucap/ng-ui';

@Component({
  selector: 'app-group-profile-list-item-02',
  templateUrl: './profile-list-item-02.component.html',
  styleUrls: ['./profile-list-item-02.component.scss'],
  animations: ucapAnimations,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem02Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoF;

  @Input()
  group: GroupDetailData;

  @Input()
  checkable = false;

  @Input()
  isMe = false;

  @Input()
  checked = false;

  @Output()
  moreMenu: EventEmitter<{
    event: MouseEvent;
    userInfo: UserInfoF;
    group: GroupDetailData;
    rect: any;
  }> = new EventEmitter();

  @Output()
  floatingProfileMenu: EventEmitter<{
    menuType: string;
    userInfo: UserInfoF;
  }> = new EventEmitter();

  @Output()
  changeCheckUser: EventEmitter<{
    checked: boolean;
    userInfo: UserInfoF;
  }> = new EventEmitter();

  @Output()
  openProfile: EventEmitter<UserInfoF> = new EventEmitter();

  @Input()
  isShow = false;
  @Input()
  isDialog = false;
  @Input()
  isClickMore = false;
  @Input()
  isMoreMenu = true;

  versionInfo2Res: VersionInfo2Response;
  authRes: AuthResponse;
  statusMsgClass: string;
  isShowStatusMsg = false;

  profileImageRoot: string;
  presenceInfo: StatusBulkInfo;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    @Self() private elementRef: ElementRef,
    private store: Store<any>
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectStatusBulkInfo, this.userInfo.seq)
      )
      .subscribe((status) => {
        this.presenceInfo = status;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        if (
          !!user &&
          !!this.userInfo &&
          String(user.info.seq) === String(this.userInfo.seq)
        ) {
          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.authResponse)
      )
      .subscribe((authRes) => {
        this.authRes = authRes;
      });

    this._userStatusMessage();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoF): void {
    this.openProfile.emit(userInfo);
  }

  onChangeCheckUser(param: { checked: boolean; userInfo: UserInfoF }) {
    this.changeCheckUser.emit(param);
    this.changeDetectorRef.detectChanges();
  }

  onMouseover(event: MouseEvent): void {
    if (!this.isMe && !this.isDialog) {
      this.isShow = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    if (!this.isMe && !this.isDialog) {
      this.isShow = false;
    }

    event.preventDefault();
    event.stopPropagation();
  }

  onClickProfileContextMenu(event: MouseEvent, menuType: string) {
    this.floatingProfileMenu.emit({ menuType, userInfo: this.userInfo });

    event.stopPropagation();
  }

  onClickMore(event: MouseEvent) {
    this.isClickMore = true;

    const rect = this.elementRef.nativeElement.getBoundingClientRect();

    this.moreMenu.emit({
      event,
      userInfo: this.userInfo,
      group: this.group,
      rect
    });

    event.stopPropagation();
  }

  _userStatusMessage(): string {
    if (
      !!this.userInfo &&
      !!this.userInfo.nickName &&
      this.userInfo.nickName.trim() !== ''
    ) {
      this.statusMsgClass = 'intro-name';
      return this.userInfo.nickName;
    } else if (
      !!this.userInfo &&
      !!this.userInfo.intro &&
      this.userInfo.intro.trim() !== ''
    ) {
      this.statusMsgClass = 'intro';
      return this.userInfo.intro;
    } else {
      // this.isShowStatusMsg = true;
      this.statusMsgClass = null;
      return '';
    }
  }
}
