import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output
} from '@angular/core';

import { select, Store } from '@ngrx/store';

import { VersionInfo2Response } from '@ucap/api-public';
import { GroupDetailData } from '@ucap/protocol-sync';
import { UserInfoSS } from '@ucap/protocol-query';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort
} from '@ucap/protocol-room';
import { StatusBulkInfo } from '@ucap/protocol-status';
import { User } from '@ucap/protocol-info';

import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import { ucapAnimations } from '@ucap/ng-ui';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;

/**
 * @Useage Chat > Room User List.
 */
@Component({
  selector: 'app-group-profile-list-item-03',
  templateUrl: './profile-list-item-03.component.html',
  styleUrls: ['./profile-list-item-03.component.scss'],
  animations: ucapAnimations,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem03Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoTypes;

  @Input()
  group: GroupDetailData;

  @Input()
  checkable = false;

  @Input()
  isMe = false;

  @Input()
  checked = false;

  @Input()
  isShow = false;

  @Input()
  isDialog = false;

  @Output()
  openProfile = new EventEmitter<string>();

  @Output()
  openChatRoom = new EventEmitter<string>();

  @Output()
  sendMessage = new EventEmitter<string>();

  @Output()
  exitForcing = new EventEmitter<UserInfoTypes>();

  user: User;
  versionInfo2Res: VersionInfo2Response;

  presenceInfo: StatusBulkInfo;

  tempRect: any;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectStatusBulkInfo, Number(this.userInfo.seq))
      )
      .subscribe((status) => {
        this.presenceInfo = status;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS): void {
    this.openProfile.emit(String(userInfo.seq));
  }

  onMouseover(event: MouseEvent): void {
    if (!this.isDialog) {
      this.isShow = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    if (!this.isDialog) {
      this.isShow = false;
    }

    event.preventDefault();
    event.stopPropagation();
  }

  onClickProfileContextMenu(event: MouseEvent, type: string) {
    event.preventDefault();
    event.stopPropagation();

    switch (type) {
      case 'PROFILE':
        {
          this.openProfile.emit(String(this.userInfo.seq));
        }
        break;
      case 'CHAT':
        {
          this.openChatRoom.emit(String(this.userInfo.seq));
        }
        break;
      case 'MESSAGE':
        {
          this.sendMessage.emit(String(this.userInfo.seq));
        }
        break;
      case 'EXIT':
        {
          this.exitForcing.emit(this.userInfo);
        }
        break;
    }
  }
}
