import { Subject } from 'rxjs';
import { takeUntil, take, map, withLatestFrom, tap } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ElementRef,
  ViewChild,
  EventEmitter,
  Output
} from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { PresenceType, StatusCode } from '@ucap/core';
import { FileUploadItem } from '@ucap/api';
import { VersionInfo2Response } from '@ucap/api-public';
import { AuthResponse, UserInfoSS } from '@ucap/protocol-query';
import { StatusBulkInfo } from '@ucap/protocol-status';
import { User } from '@ucap/protocol-info';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import {
  LoginSelector,
  AuthorizationSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';
import {
  PresenceSelector,
  UserSelector,
  CompanySelector
} from '@ucap/ng-store-organization';
import { BuddySelector } from '@ucap/ng-store-group';

import { PresenceUtil } from '@ucap/ng-ui-organization';

import { AppChatService } from '@app/services/app-chat.service';
import { UserInfoTypes } from '@app/types';
import { Company } from '@ucap/api-external';
@Component({
  selector: 'app-organization-profile-01',
  templateUrl: './profile-01.component.html',
  styleUrls: ['./profile-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Profile01Component implements OnInit, OnDestroy {
  @Input()
  set userSeq(seq: string) {
    this._userSeq = seq;
    this._refreshProfile();
  }
  get userSeq(): string {
    return this._userSeq;
  }
  _userSeq: string;

  @Output()
  openChat: EventEmitter<UserInfoSS> = new EventEmitter();
  @Output()
  sendMessage: EventEmitter<UserInfoSS> = new EventEmitter();
  @Output()
  sendCall: EventEmitter<string> = new EventEmitter();
  @Output()
  sendSms: EventEmitter<string> = new EventEmitter();
  @Output()
  createConference: EventEmitter<number> = new EventEmitter<number>();
  @Output()
  toggleFavorit: EventEmitter<{
    userInfo: UserInfoSS;
    isFavorite: boolean;
  }> = new EventEmitter();
  @Output()
  toggleBuddy: EventEmitter<{
    userInfo: UserInfoSS;
    isBuddy: boolean;
  }> = new EventEmitter();

  @Output()
  uploadProfileImage: EventEmitter<FileUploadItem> = new EventEmitter();
  @Output()
  updateIntro: EventEmitter<string> = new EventEmitter();
  @Output()
  updateNickname: EventEmitter<{
    userInfo: UserInfoTypes;
    nickname: string;
  }> = new EventEmitter();

  @ViewChild('profileImageFileInput', { static: false })
  profileImageFileInput: ElementRef<HTMLInputElement>;

  profileImageFileUploadItem: FileUploadItem;

  userInfo: UserInfoSS;

  user: User;
  authRes: AuthResponse;
  versionInfo2Res: VersionInfo2Response;
  presences: Dictionary<StatusBulkInfo>;

  String = String;
  PresenceUtil = PresenceUtil;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private queryProtocolService: QueryProtocolService,
    private appChatService: AppChatService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private i18nService: I18nService
  ) {}

  PresenceType = PresenceType;
  isBuddy = false;
  isFavorite = false;
  currentNickname: string;
  companyList: Company[];
  companyName: string;

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo2Response)
      )
      .subscribe((versionInfo2Res) => {
        this.versionInfo2Res = versionInfo2Res;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this._refreshProfile();
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.authResponse)
      )
      .subscribe((authRes) => {
        this.authRes = authRes;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectEntitiesStatusBulkInfo)
      )
      .subscribe((statusBulkInfo) => {
        this.presences = statusBulkInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(BuddySelector.buddies))
      .subscribe((buddies) => {
        const users = buddies.filter((buddy) => {
          if (!!buddy && !!this.userInfo) {
            return buddy.seq === Number(this.userInfo.seq);
          }
        });
        this.isBuddy = users.length > 0;
        if (this.isBuddy) {
          this.isFavorite = users[0].isFavorit;
          this.currentNickname = users[0].nickName;
          const profileImageFile = users[0].profileImageFile;
          const intro = users[0].intro;

          this.userInfo = { ...this.userInfo, profileImageFile, intro };
        }
        this.changeDetectorRef.markForCheck();
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        if (!companyList) {
          return;
        }
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getPresenceInfo(): StatusBulkInfo | undefined {
    if (!!this.presences) {
      return this.presences[this._userSeq];
    } else {
      return undefined;
    }
  }

  _refreshProfile() {
    if (!this.user || !this.userSeq) {
      return;
    }

    this.currentNickname = '';

    this.queryProtocolService
      .dataUser({
        divCd: 'OPENPROF',
        seq: Number(this.userSeq),
        senderCompanyCode: this.user.info.companyCode,
        senderEmployeeType: this.user.info.employeeType
      })
      .pipe(
        take(1),
        withLatestFrom(this.store.pipe(select(BuddySelector.buddies))),
        tap(([res, buddies]) => {
          this.userInfo = res.userInfo;
          const users = buddies.filter(
            (buddy) => buddy.seq === Number(this.userInfo.seq)
          );
          this.isBuddy = users.length > 0;
          if (this.isBuddy) {
            this.isFavorite = users[0].isFavorit;
            this.currentNickname = users[0].nickName.trim();
          }
          if (
            !this.userInfo.companyName ||
            (!!this.userInfo.companyName &&
              this.userInfo.companyName.trim() === '')
          ) {
            const company = this.companyList.filter(
              (companyInfo) =>
                companyInfo.companyCode === this.userInfo.companyCode
            );
            if (!!company) {
              this.companyName = company[0].companyName;
            }
          } else {
            this.companyName = this.userInfo.companyName;
          }
          this.changeDetectorRef.markForCheck();
        })
      )
      .subscribe();
  }
  onProfileImageView(userInfo: UserInfoSS) {}
  onChangeFileInput() {}
  onChangeIntro(intro: string) {
    this.updateIntro.emit(intro);
  }
  onChangeNickname(nickname: string) {
    this.updateNickname.emit({ userInfo: this.userInfo, nickname });
  }

  onToggleFavorite(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.toggleFavorit.emit(params);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    this.toggleBuddy.emit(params);
  }
  onOpenChat(userInfo: UserInfoSS) {
    this.openChat.emit(userInfo);
  }
  onSendCall(calleeNumber: string) {
    this.sendCall.emit(calleeNumber);
  }
  onSendSms(smsNum: string) {
    this.sendSms.emit(smsNum);
  }
  onCreateConference(userSeq: number) {
    this.createConference.emit(userSeq);
  }
  onSendMessage(userInfo: UserInfoSS) {
    this.sendMessage.emit(userInfo);
  }

  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    this.uploadProfileImage.emit(profileImageFileUploadItem);
  }
}
